@@ -33,18 +33,16 @@
 extern extension_entry_st ext_mod_sig;
 
 int _gnutls_session_sign_algo_requested (gnutls_session_t session,
-					 gnutls_sign_algorithm_t sig);
+                                         gnutls_sign_algorithm_t sig);
 gnutls_sign_algorithm_t _gnutls_session_get_sign_algo (gnutls_session_t
-						       session,
-						       gnutls_pk_algorithm_t
-						       pk,
-						       gnutls_digest_algorithm_t
-						       * hash);
+                                                       session,
+                                                       gnutls_pk_algorithm_t
+                                                       pk);
 int _gnutls_sign_algorithm_parse_data (gnutls_session_t session,
-				       const opaque * data, size_t data_size);
+                                       const opaque * data, size_t data_size);
 int _gnutls_sign_algorithm_write_params (gnutls_session_t session,
-					 opaque * data, size_t max_data_size);
+                                         opaque * data, size_t max_data_size);
 int _gnutls_session_sign_algo_enabled (gnutls_session_t session,
-				       gnutls_sign_algorithm_t sig);
+                                       gnutls_sign_algorithm_t sig);
 
 #endif
