@@ -33,8 +33,8 @@
 
 static void
 break_comma_list (char *etag,
-		  char **broken_etag, int *elements, int max_elements,
-		  char sep);
+                  char **broken_etag, int *elements, int max_elements,
+                  char sep);
 
 /**
  * gnutls_cipher_set_priority:
@@ -68,7 +68,9 @@ gnutls_cipher_set_priority (gnutls_session_t session, const int *list)
   return 0;
 }
 
-inline static int
+typedef void (bulk_rmadd_func) (priority_st * priority_list, const int *);
+
+inline static void
 _set_priority (priority_st * st, const int *list)
 {
   int num = 0, i;
@@ -84,8 +86,13 @@ _set_priority (priority_st * st, const int *list)
       st->priority[i] = list[i];
     }
 
-  return 0;
+  return;
+}
 
+static void
+_clear_priorities (priority_st * st, const int *list)
+{
+  memset(st, 0, sizeof(*st));  
 }
 
 /**
@@ -105,7 +112,8 @@ _set_priority (priority_st * st, const int *list)
 int
 gnutls_kx_set_priority (gnutls_session_t session, const int *list)
 {
-  return _set_priority (&session->internals.priorities.kx, list);
+  _set_priority (&session->internals.priorities.kx, list);
+  return 0;
 }
 
 /**
@@ -125,7 +133,8 @@ gnutls_kx_set_priority (gnutls_session_t session, const int *list)
 int
 gnutls_mac_set_priority (gnutls_session_t session, const int *list)
 {
-  return _set_priority (&session->internals.priorities.mac, list);
+  _set_priority (&session->internals.priorities.mac, list);
+  return 0;
 }
 
 /**
@@ -149,7 +158,8 @@ gnutls_mac_set_priority (gnutls_session_t session, const int *list)
 int
 gnutls_compression_set_priority (gnutls_session_t session, const int *list)
 {
-  return _set_priority (&session->internals.priorities.compression, list);
+  _set_priority (&session->internals.priorities.compression, list);
+  return 0;
 }
 
 /**
@@ -166,9 +176,7 @@ gnutls_compression_set_priority (gnutls_session_t session, const int *list)
 int
 gnutls_protocol_set_priority (gnutls_session_t session, const int *list)
 {
-  int ret;
-
-  ret = _set_priority (&session->internals.priorities.protocol, list);
+  _set_priority (&session->internals.priorities.protocol, list);
 
   /* set the current version to the first in the chain.
    * This will be overridden later.
@@ -176,7 +184,7 @@ gnutls_protocol_set_priority (gnutls_session_t session, const int *list)
   if (list)
     _gnutls_set_current_version (session, list[0]);
 
-  return ret;
+  return 0;
 }
 
 /**
@@ -195,11 +203,11 @@ gnutls_protocol_set_priority (gnutls_session_t session, const int *list)
  **/
 int
 gnutls_certificate_type_set_priority (gnutls_session_t session,
-				      const int *list)
+                                      const int *list)
 {
 #ifdef ENABLE_OPENPGP
-  return _set_priority (&session->internals.priorities.cert_type, list);
-
+  _set_priority (&session->internals.priorities.cert_type, list);
+  return 0;
 #else
 
   return GNUTLS_E_UNIMPLEMENTED_FEATURE;
@@ -219,11 +227,6 @@ static const int kx_priority_performance[] = {
   GNUTLS_KX_RSA,
   GNUTLS_KX_DHE_RSA,
   GNUTLS_KX_DHE_DSS,
-  GNUTLS_KX_PSK,
-  GNUTLS_KX_DHE_PSK,
-  GNUTLS_KX_SRP_RSA,
-  GNUTLS_KX_SRP_DSS,
-  GNUTLS_KX_SRP,
   /* GNUTLS_KX_ANON_DH: Man-in-the-middle prone, don't add!
    * GNUTLS_KX_RSA_EXPORT: Deprecated, don't add!
    */
@@ -234,11 +237,6 @@ static const int kx_priority_export[] = {
   GNUTLS_KX_RSA,
   GNUTLS_KX_DHE_RSA,
   GNUTLS_KX_DHE_DSS,
-  GNUTLS_KX_PSK,
-  GNUTLS_KX_DHE_PSK,
-  GNUTLS_KX_SRP_RSA,
-  GNUTLS_KX_SRP_DSS,
-  GNUTLS_KX_SRP,
   GNUTLS_KX_RSA_EXPORT,
   0
 };
@@ -249,12 +247,7 @@ static const int kx_priority_secure[] = {
    */
   GNUTLS_KX_DHE_RSA,
   GNUTLS_KX_DHE_DSS,
-  GNUTLS_KX_DHE_PSK,
-  GNUTLS_KX_SRP_RSA,
-  GNUTLS_KX_SRP_DSS,
   GNUTLS_KX_RSA,
-  GNUTLS_KX_PSK,
-  GNUTLS_KX_SRP,
   /* GNUTLS_KX_ANON_DH: Man-in-the-middle prone, don't add!
    * GNUTLS_KX_RSA_EXPORT: Deprecated, don't add!
    */
@@ -339,11 +332,13 @@ static const int comp_priority[] = {
 };
 
 static const int sign_priority_default[] = {
-  GNUTLS_SIGN_RSA_SHA1,
-  GNUTLS_SIGN_DSA_SHA1,
+  GNUTLS_SIGN_DSA_SHA224,
+  GNUTLS_SIGN_DSA_SHA256,
   GNUTLS_SIGN_RSA_SHA256,
   GNUTLS_SIGN_RSA_SHA384,
   GNUTLS_SIGN_RSA_SHA512,
+  GNUTLS_SIGN_RSA_SHA1,
+  GNUTLS_SIGN_DSA_SHA1,
   0
 };
 
@@ -362,6 +357,7 @@ static const int sign_priority_secure256[] = {
 
 static const int mac_priority_performance[] = {
   GNUTLS_MAC_SHA1,
+  GNUTLS_MAC_SHA256,
   0
 };
 
@@ -384,12 +380,12 @@ static void
 prio_remove (priority_st * priority_list, unsigned int algo)
 {
   int i = 0;
-  int pos = -1;			/* the position of the cipher to remove */
+  int pos = -1;                 /* the position of the cipher to remove */
 
   while (priority_list->priority[i] != 0)
     {
       if (priority_list->priority[i] == algo)
-	pos = i;
+        pos = i;
       i++;
     }
 
@@ -410,7 +406,7 @@ prio_add (priority_st * priority_list, unsigned int algo)
   while (priority_list->priority[i] != 0)
     {
       if (algo == priority_list->priority[i])
-	return;			/* if it exists */
+        return;                 /* if it exists */
       i++;
     }
 
@@ -444,15 +440,15 @@ gnutls_priority_set (gnutls_session_t session, gnutls_priority_t priority)
     }
 
   memcpy (&session->internals.priorities, priority,
-	  sizeof (struct gnutls_priority_st));
+          sizeof (struct gnutls_priority_st));
 
   /* set the current version to the first in the chain.
    * This will be overridden later.
    */
   if (session->internals.priorities.protocol.algorithms > 0)
     _gnutls_set_current_version (session,
-				 session->internals.priorities.protocol.
-				 priority[0]);
+                                 session->internals.priorities.protocol.
+                                 priority[0]);
 
   return 0;
 }
@@ -467,29 +463,11 @@ gnutls_priority_set (gnutls_session_t session, gnutls_priority_t priority)
  * @err_pos: In case of an error this will have the position in the string the error occured
  *
  * Sets priorities for the ciphers, key exchange methods, macs and
- * compression methods. This is to avoid using the
- * gnutls_*_priority() functions.
+ * compression methods.
  *
  * The #priorities option allows you to specify a colon
  * separated list of the cipher priorities to enable.
  *
- * Unless the first keyword is "NONE" the defaults (in preference
- * order) are for TLS protocols TLS 1.2, TLS1.1, TLS1.0, SSL3.0; for
- * compression NULL; for certificate types X.509, OpenPGP.
- *
- * For key exchange algorithms when in NORMAL or SECURE levels the
- * perfect forward secrecy algorithms take precedence of the other
- * protocols.  In all cases all the supported key exchange algorithms
- * are enabled (except for the RSA-EXPORT which is only enabled in
- * EXPORT level).
- *
- * Note that although one can select very long key sizes (such as 256 bits)
- * for symmetric algorithms, to actually increase security the public key
- * algorithms have to use longer key sizes as well.
- *
- * For all the current available algorithms and protocols use
- * "gnutls-cli -l" to get a listing.
- *
  * Common keywords: Some keywords are defined to provide quick access
  * to common preferences.
  *
@@ -518,69 +496,35 @@ gnutls_priority_set (gnutls_session_t session, gnutls_priority_t priority)
  *
  * "+" appended with an algorithm will add this algorithm.
  *
- * "%COMPAT" will enable compatibility features for a server.
- *
- * "%DISABLE_SAFE_RENEGOTIATION" will disable safe renegotiation
- * completely.  Do not use unless you know what you are doing.
- * Testing purposes only.
- *
- * "%UNSAFE_RENEGOTIATION" will allow handshakes and rehandshakes
- * without the safe renegotiation extension.  Note that for clients
- * this mode is insecure (you may be under attack), and for servers it
- * will allow insecure clients to connect (which could be fooled by an
- * attacker).  Do not use unless you know what you are doing and want
- * maximum compatibility.
- *
- * "%PARTIAL_RENEGOTIATION" will allow initial handshakes to proceed,
- * but not rehandshakes.  This leaves the client vulnerable to attack,
- * and servers will be compatible with non-upgraded clients for
- * initial handshakes.  This is currently the default for clients and
- * servers, for compatibility reasons.
- *
- * "%SAFE_RENEGOTIATION" will enforce safe renegotiation.  Clients and
- * servers will refuse to talk to an insecure peer.  Currently this
- * causes operability problems, but is required for full protection.
- *
- * "%SSL3_RECORD_VERSION" will use SSL3.0 record version in client hello.
- * This is the default.
- *
- * "%LATEST_RECORD_VERSION" will use the latest TLS version record
- * version in client hello.
- *
- * "%VERIFY_ALLOW_SIGN_RSA_MD5" will allow RSA-MD5 signatures in
- * certificate chains.
- *
- * "%VERIFY_ALLOW_X509_V1_CA_CRT" will allow V1 CAs in chains.
- *
- * Namespace concern:
- * To avoid collisions in order to specify a compression algorithm in
- * this string you have to prefix it with "COMP-", protocol versions
- * with "VERS-", signature algorithms with "SIGN-" and certificate types with "CTYPE-". All other
- * algorithms don't need a prefix. The keywords "SIGN-ALL", "CTYPE-ALL", "COMP-ALL",
- * and "VERS-TLS-ALL"  can be used to add all the support signature types, certificate
- * types, compression methods and supported TLS version numbers.
+ * Check the GnuTLS manual section "Priority strings" for detailed
+ * information.
  *
  * Examples:
- * "NORMAL:!AES-128-CBC" means normal ciphers except for AES-128.
  *
- * "EXPORT:!VERS-TLS1.0:+COMP-DEFLATE" means that export ciphers are
- * enabled, TLS 1.0 is disabled, and libz compression enabled.
+ * "NONE:+VERS-TLS-ALL:+MAC-ALL:+RSA:+AES-128-CBC:+SIGN-ALL:+COMP-NULL"
  *
- * "NONE:+VERS-TLS-ALL:+AES-128-CBC:+RSA:+SHA1:+COMP-NULL:+SIGN-RSA-SHA1", "NORMAL",
- * "%COMPAT".
+ * "NORMAL:-ARCFOUR-128" means normal ciphers except for ARCFOUR-128.
+ *
+ * "SECURE:-VERS-SSL3.0:+COMP-DEFLATE" means that only secure ciphers are
+ * enabled, SSL3.0 is disabled, and libz compression enabled.
+ *
+ * "NONE:+VERS-TLS-ALL:+AES-128-CBC:+RSA:+SHA1:+COMP-NULL:+SIGN-RSA-SHA1", 
+ *
+ * "NORMAL:%COMPAT" is the most compatible mode.
  *
  * Returns: On syntax error %GNUTLS_E_INVALID_REQUEST is returned,
  * %GNUTLS_E_SUCCESS on success, or an error code.
  **/
 int
 gnutls_priority_init (gnutls_priority_t * priority_cache,
-		      const char *priorities, const char **err_pos)
+                      const char *priorities, const char **err_pos)
 {
   char *broken_list[MAX_ELEMENTS];
   int broken_list_size = 0, i = 0, j;
   char *darg = NULL;
   int algo;
   rmadd_func *fn;
+  bulk_rmadd_func *bulk_fn;
 
   *priority_cache = gnutls_calloc (1, sizeof (struct gnutls_priority_st));
   if (*priority_cache == NULL)
@@ -624,185 +568,202 @@ gnutls_priority_init (gnutls_priority_t * priority_cache,
   for (; i < broken_list_size; i++)
     {
       if (strcasecmp (broken_list[i], "PERFORMANCE") == 0)
-	{
-	  _set_priority (&(*priority_cache)->cipher,
-			 cipher_priority_performance);
-	  _set_priority (&(*priority_cache)->kx, kx_priority_performance);
-	  _set_priority (&(*priority_cache)->mac, mac_priority_performance);
-	  _set_priority (&(*priority_cache)->sign_algo,
-			 sign_priority_default);
-	}
+        {
+          _set_priority (&(*priority_cache)->cipher,
+                         cipher_priority_performance);
+          _set_priority (&(*priority_cache)->kx, kx_priority_performance);
+          _set_priority (&(*priority_cache)->mac, mac_priority_performance);
+          _set_priority (&(*priority_cache)->sign_algo,
+                         sign_priority_default);
+        }
       else if (strcasecmp (broken_list[i], "NORMAL") == 0)
-	{
-	  _set_priority (&(*priority_cache)->cipher, cipher_priority_normal);
-	  _set_priority (&(*priority_cache)->kx, kx_priority_secure);
-	  _set_priority (&(*priority_cache)->mac, mac_priority_secure);
-	  _set_priority (&(*priority_cache)->sign_algo,
-			 sign_priority_default);
-	}
+        {
+          _set_priority (&(*priority_cache)->cipher, cipher_priority_normal);
+          _set_priority (&(*priority_cache)->kx, kx_priority_secure);
+          _set_priority (&(*priority_cache)->mac, mac_priority_secure);
+          _set_priority (&(*priority_cache)->sign_algo,
+                         sign_priority_default);
+        }
       else if (strcasecmp (broken_list[i], "SECURE256") == 0
-	       || strcasecmp (broken_list[i], "SECURE") == 0)
-	{
-	  _set_priority (&(*priority_cache)->cipher,
-			 cipher_priority_secure256);
-	  _set_priority (&(*priority_cache)->kx, kx_priority_secure);
-	  _set_priority (&(*priority_cache)->mac, mac_priority_secure);
-	  _set_priority (&(*priority_cache)->sign_algo,
-			 sign_priority_secure256);
-	}
+               || strcasecmp (broken_list[i], "SECURE") == 0)
+        {
+          _set_priority (&(*priority_cache)->cipher,
+                         cipher_priority_secure256);
+          _set_priority (&(*priority_cache)->kx, kx_priority_secure);
+          _set_priority (&(*priority_cache)->mac, mac_priority_secure);
+          _set_priority (&(*priority_cache)->sign_algo,
+                         sign_priority_secure256);
+        }
       else if (strcasecmp (broken_list[i], "SECURE128") == 0)
-	{
-	  _set_priority (&(*priority_cache)->cipher,
-			 cipher_priority_secure128);
-	  _set_priority (&(*priority_cache)->kx, kx_priority_secure);
-	  _set_priority (&(*priority_cache)->mac, mac_priority_secure);
-	  _set_priority (&(*priority_cache)->sign_algo,
-			 sign_priority_secure128);
-	}
+        {
+          _set_priority (&(*priority_cache)->cipher,
+                         cipher_priority_secure128);
+          _set_priority (&(*priority_cache)->kx, kx_priority_secure);
+          _set_priority (&(*priority_cache)->mac, mac_priority_secure);
+          _set_priority (&(*priority_cache)->sign_algo,
+                         sign_priority_secure128);
+        }
       else if (strcasecmp (broken_list[i], "EXPORT") == 0)
-	{
-	  _set_priority (&(*priority_cache)->cipher, cipher_priority_export);
-	  _set_priority (&(*priority_cache)->kx, kx_priority_export);
-	  _set_priority (&(*priority_cache)->mac, mac_priority_secure);
-	  _set_priority (&(*priority_cache)->sign_algo,
-			 sign_priority_default);
-	}			/* now check if the element is something like -ALGO */
+        {
+          _set_priority (&(*priority_cache)->cipher, cipher_priority_export);
+          _set_priority (&(*priority_cache)->kx, kx_priority_export);
+          _set_priority (&(*priority_cache)->mac, mac_priority_secure);
+          _set_priority (&(*priority_cache)->sign_algo,
+                         sign_priority_default);
+        }                       /* now check if the element is something like -ALGO */
       else if (broken_list[i][0] == '!' || broken_list[i][0] == '+'
-	       || broken_list[i][0] == '-')
-	{
-	  if (broken_list[i][0] == '+')
-	    fn = prio_add;
-	  else
-	    fn = prio_remove;
+               || broken_list[i][0] == '-')
+        {
+          if (broken_list[i][0] == '+')
+            {
+              fn = prio_add;
+              bulk_fn = _set_priority;
+            }
+          else
+            {
+              fn = prio_remove;
+              bulk_fn = _clear_priorities;
+            }
 
-	  if ((algo =
-	       gnutls_mac_get_id (&broken_list[i][1])) != GNUTLS_MAC_UNKNOWN)
-	    fn (&(*priority_cache)->mac, algo);
-	  else if ((algo = gnutls_cipher_get_id (&broken_list[i][1])) !=
-		   GNUTLS_CIPHER_UNKNOWN)
-	    fn (&(*priority_cache)->cipher, algo);
-	  else if ((algo = gnutls_kx_get_id (&broken_list[i][1])) !=
-		   GNUTLS_KX_UNKNOWN)
-	    fn (&(*priority_cache)->kx, algo);
-	  else if (strncasecmp (&broken_list[i][1], "VERS-", 5) == 0)
-	    {
-	      if (strncasecmp (&broken_list[i][1], "VERS-TLS-ALL", 12) == 0)
-		{
-		  _set_priority (&(*priority_cache)->protocol,
-				 protocol_priority);
-		}
-	      else
-		{
-		  if ((algo =
-		       gnutls_protocol_get_id (&broken_list[i][6])) !=
-		      GNUTLS_VERSION_UNKNOWN)
-		    fn (&(*priority_cache)->protocol, algo);
-		  else
-		    goto error;
-		}
-	    }			/* now check if the element is something like -ALGO */
-	  else if (strncasecmp (&broken_list[i][1], "COMP-", 5) == 0)
-	    {
-	      if (strncasecmp (&broken_list[i][1], "COMP-ALL", 8) == 0)
-		{
-		  _set_priority (&(*priority_cache)->compression,
-				 comp_priority);
-		}
-	      else
-		{
-		  if ((algo =
-		       gnutls_compression_get_id (&broken_list[i][6])) !=
-		      GNUTLS_COMP_UNKNOWN)
-		    fn (&(*priority_cache)->compression, algo);
-		  else
-		    goto error;
-		}
-	    }			/* now check if the element is something like -ALGO */
-	  else if (strncasecmp (&broken_list[i][1], "CTYPE-", 6) == 0)
-	    {
-	      if (strncasecmp (&broken_list[i][1], "CTYPE-ALL", 9) == 0)
-		{
-		  _set_priority (&(*priority_cache)->cert_type,
-				 cert_type_priority);
-		}
-	      else
-		{
-		  if ((algo =
-		       gnutls_certificate_type_get_id (&broken_list[i][7])) !=
-		      GNUTLS_CRT_UNKNOWN)
-		    fn (&(*priority_cache)->cert_type, algo);
-		  else
-		    goto error;
-		}
-	    }			/* now check if the element is something like -ALGO */
-	  else if (strncasecmp (&broken_list[i][1], "SIGN-", 5) == 0)
-	    {
-	      if (strncasecmp (&broken_list[i][1], "SIGN-ALL", 8) == 0)
-		{
-		  _set_priority (&(*priority_cache)->sign_algo,
-				 sign_priority_default);
-		}
-	      else
-		{
-		  if ((algo =
-		       gnutls_sign_get_id (&broken_list[i][6])) !=
-		      GNUTLS_SIGN_UNKNOWN)
-		    fn (&(*priority_cache)->sign_algo, algo);
-		  else
-		    goto error;
-		}
-	    }			/* now check if the element is something like -ALGO */
-	  else
-	    goto error;
-	}
+          if ((algo =
+               gnutls_mac_get_id (&broken_list[i][1])) != GNUTLS_MAC_UNKNOWN)
+            fn (&(*priority_cache)->mac, algo);
+          else if ((algo = gnutls_cipher_get_id (&broken_list[i][1])) !=
+                   GNUTLS_CIPHER_UNKNOWN)
+            fn (&(*priority_cache)->cipher, algo);
+          else if ((algo = gnutls_kx_get_id (&broken_list[i][1])) !=
+                   GNUTLS_KX_UNKNOWN)
+            fn (&(*priority_cache)->kx, algo);
+          else if (strncasecmp (&broken_list[i][1], "VERS-", 5) == 0)
+            {
+              if (strncasecmp (&broken_list[i][1], "VERS-TLS-ALL", 12) == 0)
+                {
+                  bulk_fn (&(*priority_cache)->protocol,
+                                 protocol_priority);
+                }
+              else
+                {
+                  if ((algo =
+                       gnutls_protocol_get_id (&broken_list[i][6])) !=
+                      GNUTLS_VERSION_UNKNOWN)
+                    fn (&(*priority_cache)->protocol, algo);
+                  else
+                    goto error;
+
+                }
+            }                   /* now check if the element is something like -ALGO */
+          else if (strncasecmp (&broken_list[i][1], "COMP-", 5) == 0)
+            {
+              if (strncasecmp (&broken_list[i][1], "COMP-ALL", 8) == 0)
+                {
+                  bulk_fn (&(*priority_cache)->compression,
+                                 comp_priority);
+                }
+              else
+                {
+                  if ((algo =
+                       gnutls_compression_get_id (&broken_list[i][6])) !=
+                      GNUTLS_COMP_UNKNOWN)
+                    fn (&(*priority_cache)->compression, algo);
+                  else
+                    goto error;
+                }
+            }                   /* now check if the element is something like -ALGO */
+          else if (strncasecmp (&broken_list[i][1], "CTYPE-", 6) == 0)
+            {
+              if (strncasecmp (&broken_list[i][1], "CTYPE-ALL", 9) == 0)
+                {
+                  bulk_fn (&(*priority_cache)->cert_type,
+                                 cert_type_priority);
+                }
+              else
+                {
+                  if ((algo =
+                       gnutls_certificate_type_get_id (&broken_list[i][7])) !=
+                      GNUTLS_CRT_UNKNOWN)
+                    fn (&(*priority_cache)->cert_type, algo);
+                  else
+                    goto error;
+                }
+            }                   /* now check if the element is something like -ALGO */
+          else if (strncasecmp (&broken_list[i][1], "SIGN-", 5) == 0)
+            {
+              if (strncasecmp (&broken_list[i][1], "SIGN-ALL", 8) == 0)
+                {
+                  bulk_fn (&(*priority_cache)->sign_algo,
+                                 sign_priority_default);
+                }
+              else
+                {
+                  if ((algo =
+                       gnutls_sign_get_id (&broken_list[i][6])) !=
+                      GNUTLS_SIGN_UNKNOWN)
+                    fn (&(*priority_cache)->sign_algo, algo);
+                  else
+                    goto error;
+                }
+            }
+          else if (strncasecmp (&broken_list[i][1], "MAC-ALL", 7) == 0)
+            {
+                  bulk_fn (&(*priority_cache)->mac,
+                                mac_priority_secure);
+            }
+          else if (strncasecmp (&broken_list[i][1], "CIPHER-ALL", 7) == 0)
+            {
+                  bulk_fn (&(*priority_cache)->cipher,
+                                cipher_priority_normal);
+            }
+          else
+            goto error;
+        }
       else if (broken_list[i][0] == '%')
-	{
-	  if (strcasecmp (&broken_list[i][1], "COMPAT") == 0)
-	    {
-	      (*priority_cache)->no_padding = 1;
-	      (*priority_cache)->allow_large_records = 1;
-	    }
-	  else if (strcasecmp (&broken_list[i][1],
-			       "VERIFY_ALLOW_SIGN_RSA_MD5") == 0)
-	    {
-	      prio_add (&(*priority_cache)->sign_algo, GNUTLS_SIGN_RSA_MD5);
-	      (*priority_cache)->additional_verify_flags |=
-		GNUTLS_VERIFY_ALLOW_SIGN_RSA_MD5;
-	    }
-	  else if (strcasecmp (&broken_list[i][1],
-			       "SSL3_RECORD_VERSION") == 0)
-	    (*priority_cache)->no_ssl3_record_version = 0;
-	  else if (strcasecmp (&broken_list[i][1],
-			       "LATEST_RECORD_VERSION") == 0)
-	    (*priority_cache)->no_ssl3_record_version = 1;
-	  else if (strcasecmp (&broken_list[i][1],
-			       "VERIFY_ALLOW_X509_V1_CA_CRT") == 0)
-	    (*priority_cache)->additional_verify_flags |=
-	      GNUTLS_VERIFY_ALLOW_X509_V1_CA_CRT;
-	  else if (strcasecmp (&broken_list[i][1],
-			       "UNSAFE_RENEGOTIATION") == 0)
-	    {
-	      (*priority_cache)->sr = SR_UNSAFE;
-	    }
-	  else if (strcasecmp (&broken_list[i][1], "SAFE_RENEGOTIATION") == 0)
-	    {
-	      (*priority_cache)->sr = SR_SAFE;
-	    }
-	  else if (strcasecmp (&broken_list[i][1],
-			       "PARTIAL_RENEGOTIATION") == 0)
-	    {
-	      (*priority_cache)->sr = SR_PARTIAL;
-	    }
-	  else if (strcasecmp (&broken_list[i][1],
-			       "DISABLE_SAFE_RENEGOTIATION") == 0)
-	    {
-	      (*priority_cache)->sr = SR_DISABLED;
-	    }
-	  else
-	    goto error;
-	}
+        {
+          if (strcasecmp (&broken_list[i][1], "COMPAT") == 0)
+            {
+              (*priority_cache)->no_padding = 1;
+              (*priority_cache)->allow_large_records = 1;
+            }
+          else if (strcasecmp (&broken_list[i][1],
+                               "VERIFY_ALLOW_SIGN_RSA_MD5") == 0)
+            {
+              prio_add (&(*priority_cache)->sign_algo, GNUTLS_SIGN_RSA_MD5);
+              (*priority_cache)->additional_verify_flags |=
+                GNUTLS_VERIFY_ALLOW_SIGN_RSA_MD5;
+            }
+          else if (strcasecmp (&broken_list[i][1],
+                               "SSL3_RECORD_VERSION") == 0)
+            (*priority_cache)->no_ssl3_record_version = 0;
+          else if (strcasecmp (&broken_list[i][1],
+                               "LATEST_RECORD_VERSION") == 0)
+            (*priority_cache)->no_ssl3_record_version = 1;
+          else if (strcasecmp (&broken_list[i][1],
+                               "VERIFY_ALLOW_X509_V1_CA_CRT") == 0)
+            (*priority_cache)->additional_verify_flags |=
+              GNUTLS_VERIFY_ALLOW_X509_V1_CA_CRT;
+          else if (strcasecmp (&broken_list[i][1],
+                               "UNSAFE_RENEGOTIATION") == 0)
+            {
+              (*priority_cache)->sr = SR_UNSAFE;
+            }
+          else if (strcasecmp (&broken_list[i][1], "SAFE_RENEGOTIATION") == 0)
+            {
+              (*priority_cache)->sr = SR_SAFE;
+            }
+          else if (strcasecmp (&broken_list[i][1],
+                               "PARTIAL_RENEGOTIATION") == 0)
+            {
+              (*priority_cache)->sr = SR_PARTIAL;
+            }
+          else if (strcasecmp (&broken_list[i][1],
+                               "DISABLE_SAFE_RENEGOTIATION") == 0)
+            {
+              (*priority_cache)->sr = SR_DISABLED;
+            }
+          else
+            goto error;
+        }
       else
-	goto error;
+        goto error;
     }
 
   gnutls_free (darg);
@@ -813,9 +774,9 @@ error:
     {
       *err_pos = priorities;
       for (j = 0; j < i; j++)
-	{
-	  (*err_pos) += strlen (broken_list[j]) + 1;
-	}
+        {
+          (*err_pos) += strlen (broken_list[j]) + 1;
+        }
     }
   gnutls_free (darg);
   gnutls_free (*priority_cache);
@@ -853,7 +814,7 @@ gnutls_priority_deinit (gnutls_priority_t priority_cache)
  **/
 int
 gnutls_priority_set_direct (gnutls_session_t session,
-			    const char *priorities, const char **err_pos)
+                            const char *priorities, const char **err_pos)
 {
   gnutls_priority_t prio;
   int ret;
@@ -882,8 +843,8 @@ gnutls_priority_set_direct (gnutls_session_t session,
   */
 static void
 break_comma_list (char *etag,
-		  char **broken_etag, int *elements, int max_elements,
-		  char sep)
+                  char **broken_etag, int *elements, int max_elements,
+                  char sep)
 {
   char *p = etag;
   if (sep == 0)
@@ -899,14 +860,14 @@ break_comma_list (char *etag,
 
       p = strchr (p, sep);
       if (p)
-	{
-	  *p = 0;
-	  p++;			/* move to next entry and skip white
-				 * space.
-				 */
-	  while (*p == ' ')
-	    p++;
-	}
+        {
+          *p = 0;
+          p++;                  /* move to next entry and skip white
+                                 * space.
+                                 */
+          while (*p == ' ')
+            p++;
+        }
     }
   while (p != NULL && *elements < max_elements);
 }
