@@ -30,7 +30,6 @@
 #include <gnutls_datum.h>
 #include <pkcs11_int.h>
 #include <gnutls/abstract.h>
-#include <sign.h>
 #include <gnutls_pk.h>
 #include <x509_int.h>
 #include <openpgp/openpgp_int.h>
@@ -38,6 +37,7 @@
 #include <gnutls_num.h>
 #include <x509/common.h>
 #include <x509_b64.h>
+#include <abstract_int.h>
 
 #define PK_PEM_HEADER "PUBLIC KEY"
 
@@ -45,7 +45,7 @@
 struct gnutls_pubkey_st
 {
   gnutls_pk_algorithm_t pk_algorithm;
-  unsigned int bits;		/* an indication of the security parameter */
+  unsigned int bits;            /* an indication of the security parameter */
 
   /* the size of params depends on the public
    * key algorithm
@@ -57,11 +57,25 @@ struct gnutls_pubkey_st
    *      [3] is public key
    */
   bigint_t params[MAX_PUBLIC_PARAMS_SIZE];
-  int params_size;		/* holds the size of MPI params */
+  int params_size;              /* holds the size of MPI params */
 
-  unsigned int key_usage;	/* bits from GNUTLS_KEY_* */
+  unsigned int key_usage;       /* bits from GNUTLS_KEY_* */
 };
 
+static int pubkey_to_bits(gnutls_pk_algorithm_t pk, bigint_t* params, int params_size)
+{
+  switch(pk) 
+    {
+      case GNUTLS_PK_RSA:
+        return _gnutls_mpi_get_nbits(params[0]);
+      case GNUTLS_PK_DSA:
+        if (params_size < 3) return 0;
+        return _gnutls_mpi_get_nbits(params[3]);
+      default:
+        return 0;
+    }
+}
+
 /**
  * gnutls_pubkey_get_pk_algorithm:
  * @key: should contain a #gnutls_pubkey_t structure
@@ -150,7 +164,7 @@ gnutls_pubkey_deinit (gnutls_pubkey_t key)
  **/
 int
 gnutls_pubkey_import_x509 (gnutls_pubkey_t key, gnutls_x509_crt_t crt,
-			   unsigned int flags)
+                           unsigned int flags)
 {
   int ret;
 
@@ -166,18 +180,18 @@ gnutls_pubkey_import_x509 (gnutls_pubkey_t key, gnutls_x509_crt_t crt,
     case GNUTLS_PK_RSA:
       ret = _gnutls_x509_crt_get_mpis (crt, key->params, &key->params_size);
       if (ret < 0)
-	{
-	  gnutls_assert ();
-	  return ret;
-	}
+        {
+          gnutls_assert ();
+          return ret;
+        }
       break;
     case GNUTLS_PK_DSA:
       ret = _gnutls_x509_crt_get_mpis (crt, key->params, &key->params_size);
       if (ret < 0)
-	{
-	  gnutls_assert ();
-	  return ret;
-	}
+        {
+          gnutls_assert ();
+          return ret;
+        }
 
       break;
     default:
@@ -188,6 +202,35 @@ gnutls_pubkey_import_x509 (gnutls_pubkey_t key, gnutls_x509_crt_t crt,
   return 0;
 }
 
+/**
+ * gnutls_pubkey_import_privkey:
+ * @key: The public key
+ * @pkey: The private key
+ * @usage: GNUTLS_KEY_* key usage flags.
+ * @flags: should be zero
+ *
+ * This function will import the given public key to the abstract
+ * #gnutls_pubkey_t structure.
+ *
+ * Returns: On success, %GNUTLS_E_SUCCESS is returned, otherwise a
+ *   negative error value.
+ *
+ * Since: 2.12.0
+ **/
+int
+gnutls_pubkey_import_privkey (gnutls_pubkey_t key, gnutls_privkey_t pkey,
+                              unsigned int usage, unsigned int flags)
+{
+  key->pk_algorithm = gnutls_privkey_get_pk_algorithm (pkey, &key->bits);
+
+  key->key_usage = usage;
+
+  key->params_size = sizeof (key->params) / sizeof (key->params[0]);
+
+  return _gnutls_privkey_get_public_mpis (pkey, key->params,
+                                          &key->params_size);
+}
+
 /**
  * gnutls_pubkey_get_preferred_hash_algorithm:
  * @key: Holds the certificate
@@ -205,8 +248,8 @@ gnutls_pubkey_import_x509 (gnutls_pubkey_t key, gnutls_x509_crt_t crt,
  **/
 int
 gnutls_pubkey_get_preferred_hash_algorithm (gnutls_pubkey_t key,
-					    gnutls_digest_algorithm_t *
-					    hash, unsigned int *mand)
+                                            gnutls_digest_algorithm_t *
+                                            hash, unsigned int *mand)
 {
   int ret;
 
@@ -217,8 +260,8 @@ gnutls_pubkey_get_preferred_hash_algorithm (gnutls_pubkey_t key,
     }
 
   ret = _gnutls_pk_get_hash_algorithm (key->pk_algorithm,
-				       key->params, key->params_size,
-				       hash, mand);
+                                       key->params, key->params_size,
+                                       hash, mand);
 
   return ret;
 }
@@ -238,7 +281,7 @@ gnutls_pubkey_get_preferred_hash_algorithm (gnutls_pubkey_t key,
  **/
 int
 gnutls_pubkey_import_pkcs11 (gnutls_pubkey_t key,
-			     gnutls_pkcs11_obj_t obj, unsigned int flags)
+                             gnutls_pkcs11_obj_t obj, unsigned int flags)
 {
   int ret;
 
@@ -255,12 +298,12 @@ gnutls_pubkey_import_pkcs11 (gnutls_pubkey_t key,
     {
     case GNUTLS_PK_RSA:
       ret = gnutls_pubkey_import_rsa_raw (key, &obj->pubkey[0],
-					  &obj->pubkey[1]);
+                                          &obj->pubkey[1]);
       break;
     case GNUTLS_PK_DSA:
       ret = gnutls_pubkey_import_dsa_raw (key, &obj->pubkey[0],
-					  &obj->pubkey[1],
-					  &obj->pubkey[2], &obj->pubkey[3]);
+                                          &obj->pubkey[1],
+                                          &obj->pubkey[2], &obj->pubkey[3]);
       break;
     default:
       gnutls_assert ();
@@ -291,9 +334,9 @@ gnutls_pubkey_import_pkcs11 (gnutls_pubkey_t key,
  **/
 int
 gnutls_pubkey_import_openpgp (gnutls_pubkey_t key,
-			      gnutls_openpgp_crt_t crt,
-			      gnutls_openpgp_keyid_t keyid,
-			      unsigned int flags)
+                              gnutls_openpgp_crt_t crt,
+                              gnutls_openpgp_keyid_t keyid,
+                              unsigned int flags)
 {
   int ret;
   uint32_t kid32[2];
@@ -317,23 +360,23 @@ gnutls_pubkey_import_openpgp (gnutls_pubkey_t key,
     {
     case GNUTLS_PK_RSA:
       ret =
-	_gnutls_openpgp_crt_get_mpis (crt, kid32, key->params,
-				      &key->params_size);
+        _gnutls_openpgp_crt_get_mpis (crt, kid32, key->params,
+                                      &key->params_size);
       if (ret < 0)
-	{
-	  gnutls_assert ();
-	  return ret;
-	}
+        {
+          gnutls_assert ();
+          return ret;
+        }
       break;
     case GNUTLS_PK_DSA:
       ret =
-	_gnutls_openpgp_crt_get_mpis (crt, kid32, key->params,
-				      &key->params_size);
+        _gnutls_openpgp_crt_get_mpis (crt, kid32, key->params,
+                                      &key->params_size);
       if (ret < 0)
-	{
-	  gnutls_assert ();
-	  return ret;
-	}
+        {
+          gnutls_assert ();
+          return ret;
+        }
       break;
     default:
       gnutls_assert ();
@@ -367,8 +410,8 @@ gnutls_pubkey_import_openpgp (gnutls_pubkey_t key,
  **/
 int
 gnutls_pubkey_export (gnutls_pubkey_t key,
-		      gnutls_x509_crt_fmt_t format, void *output_data,
-		      size_t * output_data_size)
+                      gnutls_x509_crt_fmt_t format, void *output_data,
+                      size_t * output_data_size)
 {
   int result;
   ASN1_TYPE spk = ASN1_TYPE_EMPTY;
@@ -389,8 +432,8 @@ gnutls_pubkey_export (gnutls_pubkey_t key,
 
   result =
     _gnutls_x509_encode_and_copy_PKI_params (spk, "",
-					     key->pk_algorithm,
-					     key->params, key->params_size);
+                                             key->pk_algorithm,
+                                             key->params, key->params_size);
   if (result < 0)
     {
       gnutls_assert ();
@@ -398,8 +441,8 @@ gnutls_pubkey_export (gnutls_pubkey_t key,
     }
 
   result = _gnutls_x509_export_int_named (spk, "",
-					  format, PK_PEM_HEADER,
-					  output_data, output_data_size);
+                                          format, PK_PEM_HEADER,
+                                          output_data, output_data_size);
   if (result < 0)
     {
       gnutls_assert ();
@@ -437,8 +480,8 @@ cleanup:
  **/
 int
 gnutls_pubkey_get_key_id (gnutls_pubkey_t key, unsigned int flags,
-			  unsigned char *output_data,
-			  size_t * output_data_size)
+                          unsigned char *output_data,
+                          size_t * output_data_size)
 {
   int ret = 0;
 
@@ -450,7 +493,7 @@ gnutls_pubkey_get_key_id (gnutls_pubkey_t key, unsigned int flags,
 
   ret =
     _gnutls_get_key_id (key->pk_algorithm, key->params,
-			key->params_size, output_data, output_data_size);
+                        key->params_size, output_data, output_data_size);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -474,7 +517,7 @@ gnutls_pubkey_get_key_id (gnutls_pubkey_t key, unsigned int flags,
  **/
 int
 gnutls_pubkey_get_pk_rsa_raw (gnutls_pubkey_t key,
-			      gnutls_datum_t * m, gnutls_datum_t * e)
+                              gnutls_datum_t * m, gnutls_datum_t * e)
 {
   int ret;
 
@@ -524,8 +567,8 @@ gnutls_pubkey_get_pk_rsa_raw (gnutls_pubkey_t key,
  **/
 int
 gnutls_pubkey_get_pk_dsa_raw (gnutls_pubkey_t key,
-			      gnutls_datum_t * p, gnutls_datum_t * q,
-			      gnutls_datum_t * g, gnutls_datum_t * y)
+                              gnutls_datum_t * p, gnutls_datum_t * q,
+                              gnutls_datum_t * g, gnutls_datum_t * y)
 {
   int ret;
 
@@ -599,8 +642,8 @@ gnutls_pubkey_get_pk_dsa_raw (gnutls_pubkey_t key,
  **/
 int
 gnutls_pubkey_import (gnutls_pubkey_t key,
-		      const gnutls_datum_t * data,
-		      gnutls_x509_crt_fmt_t format)
+                      const gnutls_datum_t * data,
+                      gnutls_x509_crt_fmt_t format)
 {
   int result = 0, need_free = 0;
   gnutls_datum_t _data;
@@ -623,15 +666,15 @@ gnutls_pubkey_import (gnutls_pubkey_t key,
 
       /* Try the first header */
       result =
-	_gnutls_fbase64_decode (PK_PEM_HEADER, data->data, data->size, &out);
+        _gnutls_fbase64_decode (PK_PEM_HEADER, data->data, data->size, &out);
 
       if (result <= 0)
-	{
-	  if (result == 0)
-	    result = GNUTLS_E_INTERNAL_ERROR;
-	  gnutls_assert ();
-	  return result;
-	}
+        {
+          if (result == 0)
+            result = GNUTLS_E_INTERNAL_ERROR;
+          gnutls_assert ();
+          return result;
+        }
 
       _data.data = out;
       _data.size = result;
@@ -668,6 +711,7 @@ gnutls_pubkey_import (gnutls_pubkey_t key,
    * fail.
    */
   key->pk_algorithm = _gnutls_x509_get_pk_algorithm (spk, "", NULL);
+  key->bits = pubkey_to_bits(key->pk_algorithm, key->params, key->params_size);
 
   result = 0;
 
@@ -702,10 +746,10 @@ gnutls_x509_crt_set_pubkey (gnutls_x509_crt_t crt, gnutls_pubkey_t key)
     }
 
   result = _gnutls_x509_encode_and_copy_PKI_params (crt->cert,
-						    "tbsCertificate.subjectPublicKeyInfo",
-						    key->pk_algorithm,
-						    key->params,
-						    key->params_size);
+                                                    "tbsCertificate.subjectPublicKeyInfo",
+                                                    key->pk_algorithm,
+                                                    key->params,
+                                                    key->params_size);
 
   if (result < 0)
     {
@@ -763,7 +807,9 @@ gnutls_x509_crq_set_pubkey (gnutls_x509_crq_t crq, gnutls_pubkey_t key)
  * @crt: a certificate of type #gnutls_x509_crt_t
  * @usage: an ORed sequence of the GNUTLS_KEY_* elements.
  *
- * This function will set the keyUsage certificate extension.
+ * This function will set the key usage flags of the public key. This
+ * is only useful if the key is to be exported to a certificate or
+ * certificate request.
  *
  * Returns: On success, %GNUTLS_E_SUCCESS is returned, otherwise a
  *   negative error value.
@@ -791,7 +837,7 @@ gnutls_pubkey_set_key_usage (gnutls_pubkey_t key, unsigned int usage)
 
 int
 gnutls_pubkey_import_pkcs11_url (gnutls_pubkey_t key, const char *url,
-				 unsigned int flags)
+                                 unsigned int flags)
 {
   gnutls_pkcs11_obj_t pcrt;
   int ret;
@@ -839,8 +885,8 @@ cleanup:
  **/
 int
 gnutls_pubkey_import_rsa_raw (gnutls_pubkey_t key,
-			      const gnutls_datum_t * m,
-			      const gnutls_datum_t * e)
+                              const gnutls_datum_t * m,
+                              const gnutls_datum_t * e)
 {
   size_t siz = 0;
 
@@ -867,6 +913,7 @@ gnutls_pubkey_import_rsa_raw (gnutls_pubkey_t key,
 
   key->params_size = RSA_PUBLIC_PARAMS;
   key->pk_algorithm = GNUTLS_PK_RSA;
+  key->bits = pubkey_to_bits(GNUTLS_PK_RSA, key->params, key->params_size);
 
   return 0;
 }
@@ -888,10 +935,10 @@ gnutls_pubkey_import_rsa_raw (gnutls_pubkey_t key,
  **/
 int
 gnutls_pubkey_import_dsa_raw (gnutls_pubkey_t key,
-			      const gnutls_datum_t * p,
-			      const gnutls_datum_t * q,
-			      const gnutls_datum_t * g,
-			      const gnutls_datum_t * y)
+                              const gnutls_datum_t * p,
+                              const gnutls_datum_t * q,
+                              const gnutls_datum_t * g,
+                              const gnutls_datum_t * y)
 {
   size_t siz = 0;
 
@@ -937,11 +984,52 @@ gnutls_pubkey_import_dsa_raw (gnutls_pubkey_t key,
 
   key->params_size = DSA_PUBLIC_PARAMS;
   key->pk_algorithm = GNUTLS_PK_DSA;
+  key->bits = pubkey_to_bits(GNUTLS_PK_DSA, key->params, key->params_size);
 
   return 0;
 
 }
 
+/**
+ * gnutls_pubkey_verify_data:
+ * @pubkey: Holds the public key
+ * @flags: should be 0 for now
+ * @data: holds the data to be signed
+ * @signature: contains the signature
+ *
+ * This function will verify the given signed data, using the
+ * parameters from the certificate.
+ *
+ * Returns: In case of a verification failure
+ *   %GNUTLS_E_PK_SIG_VERIFY_FAILED is returned, and a positive code
+ *   on success.
+ *
+ * Since: 2.12.0
+ **/
+int
+gnutls_pubkey_verify_data (gnutls_pubkey_t pubkey, unsigned int flags,
+			   const gnutls_datum_t * data,
+			   const gnutls_datum_t * signature)
+{
+  int ret;
+
+  if (pubkey == NULL)
+    {
+      gnutls_assert ();
+      return GNUTLS_E_INVALID_REQUEST;
+    }
+
+  ret = pubkey_verify_sig( data, NULL, signature, pubkey->pk_algorithm, 
+    pubkey->params, pubkey->params_size);
+  if (ret < 0)
+    {
+      gnutls_assert();
+    }
+
+  return ret;
+}
+
+
 /**
  * gnutls_pubkey_verify_hash:
  * @key: Holds the certificate
@@ -952,13 +1040,13 @@ gnutls_pubkey_import_dsa_raw (gnutls_pubkey_t key,
  * This function will verify the given signed digest, using the
  * parameters from the certificate.
  *
- * Returns: In case of a verification failure 0 is returned, and 1 on
- * success.
+ * Returns: In case of a verification failure %GNUTLS_E_PK_SIG_VERIFY_FAILED 
+ * is returned, and a positive code on success.
  **/
 int
 gnutls_pubkey_verify_hash (gnutls_pubkey_t key, unsigned int flags,
-			   const gnutls_datum_t * hash,
-			   const gnutls_datum_t * signature)
+                           const gnutls_datum_t * hash,
+                           const gnutls_datum_t * signature)
 {
   int ret;
 
@@ -970,7 +1058,7 @@ gnutls_pubkey_verify_hash (gnutls_pubkey_t key, unsigned int flags,
 
   ret =
     pubkey_verify_sig (NULL, hash, signature, key->pk_algorithm,
-		       key->params, key->params_size);
+                       key->params, key->params_size);
 
   return ret;
 }
@@ -989,8 +1077,8 @@ gnutls_pubkey_verify_hash (gnutls_pubkey_t key, unsigned int flags,
  **/
 int
 gnutls_pubkey_get_verify_algorithm (gnutls_pubkey_t key,
-				    const gnutls_datum_t * signature,
-				    gnutls_digest_algorithm_t * hash)
+                                    const gnutls_datum_t * signature,
+                                    gnutls_digest_algorithm_t * hash)
 {
   if (key == NULL)
     {
@@ -999,8 +1087,8 @@ gnutls_pubkey_get_verify_algorithm (gnutls_pubkey_t key,
     }
 
   return _gnutls_x509_verify_algorithm ((gnutls_mac_algorithm_t *)
-					hash, signature,
-					key->pk_algorithm,
-					key->params, key->params_size);
+                                        hash, signature,
+                                        key->pk_algorithm,
+                                        key->params, key->params_size);
 
 }
