@@ -43,10 +43,10 @@
 #include <x509/common.h>
 
 static int
-_gnutls_tls_sign (gnutls_session_t session,
-		  gnutls_cert * cert, gnutls_privkey_t pkey,
-		  const gnutls_datum_t * hash_concat,
-		  gnutls_datum_t * signature);
+sign_tls_hash (gnutls_session_t session, gnutls_digest_algorithm_t hash_algo,
+                  gnutls_cert * cert, gnutls_privkey_t pkey,
+                  const gnutls_datum_t * hash_concat,
+                  gnutls_datum_t * signature);
 
 /* While this is currently equal to the length of RSA/SHA512
  * signature, it should also be sufficient for DSS signature and any
@@ -55,82 +55,46 @@ _gnutls_tls_sign (gnutls_session_t session,
  */
 #define MAX_SIG_SIZE 19 + MAX_HASH_SIZE
 
-/* Create a DER-encoded value as a opaque signature when RSA is used.
- * See RFC 5246 DigitallySigned for the actual format.
- */
-static int
-_gnutls_rsa_encode_sig (gnutls_mac_algorithm_t algo,
-			const gnutls_datum_t * hash,
-			gnutls_datum_t * signature)
+static int 
+get_hash_algo(gnutls_session_t session, int version,
+  gnutls_cert* cert, 
+  gnutls_sign_algorithm_t sign_algo,
+  gnutls_digest_algorithm_t *hash_algo)
 {
-  ASN1_TYPE di;
-  const char *oid;
-  int result, signature_size;
+int ret;
 
-  oid = _gnutls_x509_mac_to_oid (algo);
-  if (!oid)
-    {
-      gnutls_assert ();
-      return GNUTLS_E_UNKNOWN_HASH_ALGORITHM;
+  if (cert->subject_pk_algorithm == GNUTLS_PK_DSA)
+    { /* override */
+      if (!_gnutls_version_has_selectable_sighash (version))
+        *hash_algo = GNUTLS_DIG_SHA1;
+      else
+        {
+          *hash_algo = _gnutls_dsa_q_to_hash (cert->params[1]);
+
+          ret = _gnutls_session_sign_algo_requested(session, _gnutls_x509_pk_to_sign (GNUTLS_PK_DSA, *hash_algo));
+          if (ret < 0)
+            return gnutls_assert_val(ret);
+        }
     }
-
-  if ((result = asn1_create_element
-       (_gnutls_get_gnutls_asn (), "GNUTLS.DigestInfo", &di)) != ASN1_SUCCESS)
+  else
     {
-      gnutls_assert ();
-      return _gnutls_asn2err (result);
+      if (sign_algo == GNUTLS_SIGN_UNKNOWN)
+        *hash_algo = GNUTLS_DIG_SHA1;
+      else
+        *hash_algo = _gnutls_sign_get_hash_algorithm (sign_algo);
     }
 
-  if ((result = asn1_write_value (di, "digestAlgorithm.algorithm",
-				  oid, strlen (oid))) != ASN1_SUCCESS)
-    {
-      gnutls_assert ();
-      asn1_delete_structure (&di);
-      return _gnutls_asn2err (result);
-    }
-
-  /* Use NULL parameters. */
-  if ((result = asn1_write_value (di, "digestAlgorithm.parameters",
-				  ASN1_NULL, ASN1_NULL_SIZE)) != ASN1_SUCCESS)
-    {
-      gnutls_assert ();
-      asn1_delete_structure (&di);
-      return _gnutls_asn2err (result);
-    }
-
-  if ((result = asn1_write_value (di, "digest",
-				  hash->data, hash->size)) != ASN1_SUCCESS)
-    {
-      gnutls_assert ();
-      asn1_delete_structure (&di);
-      return _gnutls_asn2err (result);
-    }
-
-  signature_size = signature->size;
-  result = asn1_der_coding (di, "", signature->data, &signature_size, NULL);
-  asn1_delete_structure (&di);
-
-  if (result != ASN1_SUCCESS)
-    {
-      gnutls_assert ();
-      return _gnutls_asn2err (result);
-    }
-
-  signature->size = signature_size;
-
   return 0;
 }
 
-
-
 /* Generates a signature of all the random data and the parameters.
  * Used in DHE_* ciphersuites.
  */
 int
 _gnutls_handshake_sign_data (gnutls_session_t session, gnutls_cert * cert,
-			     gnutls_privkey_t pkey, gnutls_datum_t * params,
-			     gnutls_datum_t * signature,
-			     gnutls_sign_algorithm_t * sign_algo)
+                             gnutls_privkey_t pkey, gnutls_datum_t * params,
+                             gnutls_datum_t * signature,
+                             gnutls_sign_algorithm_t * sign_algo)
 {
   gnutls_datum_t dconcat;
   int ret;
@@ -140,14 +104,17 @@ _gnutls_handshake_sign_data (gnutls_session_t session, gnutls_cert * cert,
   gnutls_digest_algorithm_t hash_algo;
 
   *sign_algo =
-    _gnutls_session_get_sign_algo (session, cert->subject_pk_algorithm,
-				   &hash_algo);
+    _gnutls_session_get_sign_algo (session, cert->subject_pk_algorithm);
   if (*sign_algo == GNUTLS_SIGN_UNKNOWN)
     {
       gnutls_assert ();
       return GNUTLS_E_UNKNOWN_PK_ALGORITHM;
     }
 
+  ret = get_hash_algo(session, ver, cert, *sign_algo, &hash_algo);
+  if (ret < 0)
+    return gnutls_assert_val(ret);
+
   ret = _gnutls_hash_init (&td_sha, hash_algo);
   if (ret < 0)
     {
@@ -156,60 +123,55 @@ _gnutls_handshake_sign_data (gnutls_session_t session, gnutls_cert * cert,
     }
 
   _gnutls_hash (&td_sha, session->security_parameters.client_random,
-		GNUTLS_RANDOM_SIZE);
+                GNUTLS_RANDOM_SIZE);
   _gnutls_hash (&td_sha, session->security_parameters.server_random,
-		GNUTLS_RANDOM_SIZE);
+                GNUTLS_RANDOM_SIZE);
   _gnutls_hash (&td_sha, params->data, params->size);
 
   switch (cert->subject_pk_algorithm)
     {
     case GNUTLS_PK_RSA:
-      if (!_gnutls_version_has_selectable_prf (ver))
-	{
-	  digest_hd_st td_md5;
+      if (!_gnutls_version_has_selectable_sighash (ver))
+        {
+          digest_hd_st td_md5;
 
-	  ret = _gnutls_hash_init (&td_md5, GNUTLS_MAC_MD5);
-	  if (ret < 0)
-	    {
-	      gnutls_assert ();
-	      return ret;
-	    }
+          ret = _gnutls_hash_init (&td_md5, GNUTLS_MAC_MD5);
+          if (ret < 0)
+            {
+              gnutls_assert ();
+              return ret;
+            }
 
-	  _gnutls_hash (&td_md5, session->security_parameters.client_random,
-			GNUTLS_RANDOM_SIZE);
-	  _gnutls_hash (&td_md5, session->security_parameters.server_random,
-			GNUTLS_RANDOM_SIZE);
-	  _gnutls_hash (&td_md5, params->data, params->size);
+          _gnutls_hash (&td_md5, session->security_parameters.client_random,
+                        GNUTLS_RANDOM_SIZE);
+          _gnutls_hash (&td_md5, session->security_parameters.server_random,
+                        GNUTLS_RANDOM_SIZE);
+          _gnutls_hash (&td_md5, params->data, params->size);
 
-	  _gnutls_hash_deinit (&td_md5, concat);
-	  _gnutls_hash_deinit (&td_sha, &concat[16]);
+          _gnutls_hash_deinit (&td_md5, concat);
+          _gnutls_hash_deinit (&td_sha, &concat[16]);
 
-	  dconcat.data = concat;
-	  dconcat.size = 36;
-	}
+          dconcat.data = concat;
+          dconcat.size = 36;
+        }
       else
-	{			/* TLS 1.2 way */
-	  gnutls_datum_t hash;
+        { /* TLS 1.2 way */
 
-	  _gnutls_hash_deinit (&td_sha, concat);
+          _gnutls_hash_deinit (&td_sha, concat);
 
-	  hash.data = concat;
-	  hash.size = _gnutls_hash_get_algo_len (hash_algo);
-	  dconcat.data = concat;
-	  dconcat.size = sizeof concat;
-
-	  _gnutls_rsa_encode_sig (hash_algo, &hash, &dconcat);
-	}
+          dconcat.data = concat;
+          dconcat.size = _gnutls_hash_get_algo_len (hash_algo);
+        }
       break;
     case GNUTLS_PK_DSA:
       _gnutls_hash_deinit (&td_sha, concat);
 
       if ((hash_algo != GNUTLS_DIG_SHA1) && (hash_algo != GNUTLS_DIG_SHA224)
-	  && (hash_algo != GNUTLS_DIG_SHA256))
-	{
-	  gnutls_assert ();
-	  return GNUTLS_E_INTERNAL_ERROR;
-	}
+          && (hash_algo != GNUTLS_DIG_SHA256))
+        {
+          gnutls_assert ();
+          return GNUTLS_E_INTERNAL_ERROR;
+        }
       dconcat.data = concat;
       dconcat.size = _gnutls_hash_get_algo_len (hash_algo);
       break;
@@ -219,7 +181,8 @@ _gnutls_handshake_sign_data (gnutls_session_t session, gnutls_cert * cert,
       _gnutls_hash_deinit (&td_sha, NULL);
       return GNUTLS_E_INTERNAL_ERROR;
     }
-  ret = _gnutls_tls_sign (session, cert, pkey, &dconcat, signature);
+
+  ret = sign_tls_hash (session, hash_algo, cert, pkey, &dconcat, signature);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -235,8 +198,8 @@ _gnutls_handshake_sign_data (gnutls_session_t session, gnutls_cert * cert,
  */
 int
 _gnutls_soft_sign (gnutls_pk_algorithm_t algo, bigint_t * params,
-		   int params_size, const gnutls_datum_t * data,
-		   gnutls_datum_t * signature)
+                   int params_size, const gnutls_datum_t * data,
+                   gnutls_datum_t * signature)
 {
   int ret;
 
@@ -245,20 +208,20 @@ _gnutls_soft_sign (gnutls_pk_algorithm_t algo, bigint_t * params,
     case GNUTLS_PK_RSA:
       /* encrypt */
       if ((ret = _gnutls_pkcs1_rsa_encrypt (signature, data, params,
-					    params_size, 1)) < 0)
-	{
-	  gnutls_assert ();
-	  return ret;
-	}
+                                            params_size, 1)) < 0)
+        {
+          gnutls_assert ();
+          return ret;
+        }
 
       break;
     case GNUTLS_PK_DSA:
       /* sign */
       if ((ret = _gnutls_dsa_sign (signature, data, params, params_size)) < 0)
-	{
-	  gnutls_assert ();
-	  return ret;
-	}
+        {
+          gnutls_assert ();
+          return ret;
+        }
       break;
     default:
       gnutls_assert ();
@@ -274,11 +237,12 @@ _gnutls_soft_sign (gnutls_pk_algorithm_t algo, bigint_t * params,
  * it supports signing.
  */
 static int
-_gnutls_tls_sign (gnutls_session_t session,
-		  gnutls_cert * cert, gnutls_privkey_t pkey,
-		  const gnutls_datum_t * hash_concat,
-		  gnutls_datum_t * signature)
+sign_tls_hash (gnutls_session_t session, gnutls_digest_algorithm_t hash_algo,
+                  gnutls_cert * cert, gnutls_privkey_t pkey,
+                  const gnutls_datum_t * hash_concat,
+                  gnutls_datum_t * signature)
 {
+  gnutls_protocol_t ver = gnutls_protocol_get_version (session);
 
   /* If our certificate supports signing
    */
@@ -286,40 +250,43 @@ _gnutls_tls_sign (gnutls_session_t session,
   if (cert != NULL)
     {
       if (cert->key_usage != 0)
-	if (!(cert->key_usage & KEY_DIGITAL_SIGNATURE))
-	  {
-	    gnutls_assert ();
-	    return GNUTLS_E_KEY_USAGE_VIOLATION;
-	  }
+        if (!(cert->key_usage & GNUTLS_KEY_DIGITAL_SIGNATURE))
+          {
+            gnutls_assert ();
+            return GNUTLS_E_KEY_USAGE_VIOLATION;
+          }
 
       /* External signing. */
       if (!pkey)
-	{
-	  if (!session->internals.sign_func)
-	    return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
+        {
+          if (!session->internals.sign_func)
+            return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
 
-	  return (*session->internals.sign_func)
-	    (session, session->internals.sign_func_userdata,
-	     cert->cert_type, &cert->raw, hash_concat, signature);
-	}
+          return (*session->internals.sign_func)
+            (session, session->internals.sign_func_userdata,
+             cert->cert_type, &cert->raw, hash_concat, signature);
+        }
     }
 
-  return gnutls_privkey_sign_hash (pkey, hash_concat, signature);
+   if (!_gnutls_version_has_selectable_sighash (ver))
+    return _gnutls_privkey_sign_hash (pkey, hash_concat, signature);
+  else
+    return gnutls_privkey_sign_hash (pkey, hash_algo, 0, hash_concat, signature);
 }
 
 static int
-_gnutls_verify_sig (gnutls_cert * cert,
-		    const gnutls_datum_t * hash_concat,
-		    gnutls_datum_t * signature, size_t sha1pos,
-		    gnutls_pk_algorithm_t pk_algo)
+verify_tls_hash (gnutls_protocol_t ver, gnutls_cert * cert,
+                    const gnutls_datum_t * hash_concat,
+                    gnutls_datum_t * signature, size_t sha1pos,
+                    gnutls_pk_algorithm_t pk_algo)
 {
   int ret;
   gnutls_datum_t vdata;
 
   if (cert == NULL || cert->version == 0)
-    {				/* this is the only way to check
-				 * if it is initialized
-				 */
+    {                           /* this is the only way to check
+                                 * if it is initialized
+                                 */
       gnutls_assert ();
       return GNUTLS_E_CERTIFICATE_ERROR;
     }
@@ -327,10 +294,10 @@ _gnutls_verify_sig (gnutls_cert * cert,
   /* If the certificate supports signing continue.
    */
   if (cert->key_usage != 0)
-    if (!(cert->key_usage & KEY_DIGITAL_SIGNATURE))
+    if (!(cert->key_usage & GNUTLS_KEY_DIGITAL_SIGNATURE))
       {
-	gnutls_assert ();
-	return GNUTLS_E_KEY_USAGE_VIOLATION;
+        gnutls_assert ();
+        return GNUTLS_E_KEY_USAGE_VIOLATION;
       }
 
   if (pk_algo == GNUTLS_PK_UNKNOWN)
@@ -343,12 +310,18 @@ _gnutls_verify_sig (gnutls_cert * cert,
       vdata.size = hash_concat->size;
 
       /* verify signature */
-      if ((ret = _gnutls_rsa_verify (&vdata, signature, cert->params,
-				     cert->params_size, 1)) < 0)
-	{
-	  gnutls_assert ();
-	  return ret;
-	}
+      if (!_gnutls_version_has_selectable_sighash (ver))
+        ret = _gnutls_rsa_verify (&vdata, signature, cert->params,
+                                     cert->params_size, 1);
+      else
+        ret = pubkey_verify_sig( NULL, &vdata, signature, pk_algo, 
+          cert->params, cert->params_size);
+
+      if (ret < 0)
+        {
+          gnutls_assert ();
+          return ret;
+        }
 
       break;
     case GNUTLS_PK_DSA:
@@ -356,16 +329,16 @@ _gnutls_verify_sig (gnutls_cert * cert,
       vdata.data = &hash_concat->data[sha1pos];
       vdata.size = hash_concat->size - sha1pos;
 
+      ret = pubkey_verify_sig( NULL, &vdata, signature, pk_algo, 
+        cert->params, cert->params_size);
       /* verify signature */
-      if ((ret = _gnutls_dsa_verify (&vdata, signature, cert->params,
-				     cert->params_size)) < 0)
-	{
-	  gnutls_assert ();
-	  return ret;
-	}
+      if (ret < 0)
+        {
+          gnutls_assert ();
+          return ret;
+        }
 
       break;
-
     default:
       gnutls_assert ();
       return GNUTLS_E_INTERNAL_ERROR;
@@ -382,9 +355,9 @@ _gnutls_verify_sig (gnutls_cert * cert,
  */
 int
 _gnutls_handshake_verify_data (gnutls_session_t session, gnutls_cert * cert,
-			       const gnutls_datum_t * params,
-			       gnutls_datum_t * signature,
-			       gnutls_sign_algorithm_t algo)
+                               const gnutls_datum_t * params,
+                               gnutls_datum_t * signature,
+                               gnutls_sign_algorithm_t algo)
 {
   gnutls_datum_t dconcat;
   int ret;
@@ -392,7 +365,7 @@ _gnutls_handshake_verify_data (gnutls_session_t session, gnutls_cert * cert,
   digest_hd_st td_sha;
   opaque concat[MAX_SIG_SIZE];
   gnutls_protocol_t ver = gnutls_protocol_get_version (session);
-  gnutls_digest_algorithm_t hash_algo = GNUTLS_DIG_SHA1;
+  gnutls_digest_algorithm_t hash_algo;
 
   ret = _gnutls_session_sign_algo_enabled (session, algo);
   if (ret < 0)
@@ -401,41 +374,42 @@ _gnutls_handshake_verify_data (gnutls_session_t session, gnutls_cert * cert,
       return ret;
     }
 
-  if (!_gnutls_version_has_selectable_prf (ver))
+  if (!_gnutls_version_has_selectable_sighash (ver))
     {
       ret = _gnutls_hash_init (&td_md5, GNUTLS_MAC_MD5);
       if (ret < 0)
-	{
-	  gnutls_assert ();
-	  return ret;
-	}
+        {
+          gnutls_assert ();
+          return ret;
+        }
 
       _gnutls_hash (&td_md5, session->security_parameters.client_random,
-		    GNUTLS_RANDOM_SIZE);
+                    GNUTLS_RANDOM_SIZE);
       _gnutls_hash (&td_md5, session->security_parameters.server_random,
-		    GNUTLS_RANDOM_SIZE);
+                    GNUTLS_RANDOM_SIZE);
       _gnutls_hash (&td_md5, params->data, params->size);
     }
 
-  if (algo != GNUTLS_SIGN_UNKNOWN)
-    hash_algo = _gnutls_sign_get_hash_algorithm (algo);
+  ret = get_hash_algo(session, ver, cert, algo, &hash_algo);
+  if (ret < 0)
+    return gnutls_assert_val(ret);
 
   ret = _gnutls_hash_init (&td_sha, hash_algo);
   if (ret < 0)
     {
       gnutls_assert ();
-      if (!_gnutls_version_has_selectable_prf (ver))
-	_gnutls_hash_deinit (&td_md5, NULL);
+      if (!_gnutls_version_has_selectable_sighash (ver))
+        _gnutls_hash_deinit (&td_md5, NULL);
       return ret;
     }
 
   _gnutls_hash (&td_sha, session->security_parameters.client_random,
-		GNUTLS_RANDOM_SIZE);
+                GNUTLS_RANDOM_SIZE);
   _gnutls_hash (&td_sha, session->security_parameters.server_random,
-		GNUTLS_RANDOM_SIZE);
+                GNUTLS_RANDOM_SIZE);
   _gnutls_hash (&td_sha, params->data, params->size);
 
-  if (!_gnutls_version_has_selectable_prf (ver))
+  if (!_gnutls_version_has_selectable_sighash (ver))
     {
       _gnutls_hash_deinit (&td_md5, concat);
       _gnutls_hash_deinit (&td_sha, &concat[16]);
@@ -444,22 +418,16 @@ _gnutls_handshake_verify_data (gnutls_session_t session, gnutls_cert * cert,
     }
   else
     {
-      gnutls_datum_t hash;
-
       _gnutls_hash_deinit (&td_sha, concat);
 
-      hash.data = concat;
-      hash.size = _gnutls_hash_get_algo_len (hash_algo);
       dconcat.data = concat;
-      dconcat.size = sizeof concat;
-
-      _gnutls_rsa_encode_sig (hash_algo, &hash, &dconcat);
+      dconcat.size = _gnutls_hash_get_algo_len (hash_algo);
     }
 
-  ret = _gnutls_verify_sig (cert, &dconcat, signature,
-			    dconcat.size -
-			    _gnutls_hash_get_algo_len (hash_algo),
-			    _gnutls_sign_get_pk_algorithm (algo));
+  ret = verify_tls_hash (ver, cert, &dconcat, signature,
+                            dconcat.size -
+                            _gnutls_hash_get_algo_len (hash_algo),
+                            _gnutls_sign_get_pk_algorithm (algo));
   if (ret < 0)
     {
       gnutls_assert ();
@@ -477,9 +445,9 @@ _gnutls_handshake_verify_data (gnutls_session_t session, gnutls_cert * cert,
  */
 static int
 _gnutls_handshake_verify_cert_vrfy12 (gnutls_session_t session,
-				      gnutls_cert * cert,
-				      gnutls_datum_t * signature,
-				      gnutls_sign_algorithm_t sign_algo)
+                                      gnutls_cert * cert,
+                                      gnutls_datum_t * signature,
+                                      gnutls_sign_algorithm_t sign_algo)
 {
   int ret;
   opaque concat[MAX_SIG_SIZE];
@@ -488,6 +456,7 @@ _gnutls_handshake_verify_cert_vrfy12 (gnutls_session_t session,
   gnutls_sign_algorithm_t _sign_algo;
   gnutls_digest_algorithm_t hash_algo;
   digest_hd_st *handshake_td;
+  gnutls_protocol_t ver = gnutls_protocol_get_version (session);
 
   handshake_td = &session->internals.handshake_mac_handle.tls12.sha1;
   hash_algo = handshake_td->algorithm;
@@ -499,12 +468,12 @@ _gnutls_handshake_verify_cert_vrfy12 (gnutls_session_t session,
       handshake_td = &session->internals.handshake_mac_handle.tls12.sha256;
       hash_algo = handshake_td->algorithm;
       _sign_algo =
-	_gnutls_x509_pk_to_sign (cert->subject_pk_algorithm, hash_algo);
+        _gnutls_x509_pk_to_sign (cert->subject_pk_algorithm, hash_algo);
       if (sign_algo != _sign_algo)
-	{
-	  gnutls_assert ();
-	  return GNUTLS_E_UNSUPPORTED_SIGNATURE_ALGORITHM;
-	}
+        {
+          gnutls_assert ();
+          return GNUTLS_E_UNSUPPORTED_SIGNATURE_ALGORITHM;
+        }
     }
 
   ret = _gnutls_hash_copy (&td, handshake_td);
@@ -520,8 +489,8 @@ _gnutls_handshake_verify_cert_vrfy12 (gnutls_session_t session,
   dconcat.size = _gnutls_hash_get_algo_len (hash_algo);
 
   ret =
-    _gnutls_verify_sig (cert, &dconcat, signature, 0,
-			cert->subject_pk_algorithm);
+    verify_tls_hash (ver, cert, &dconcat, signature, 0,
+                        cert->subject_pk_algorithm);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -537,9 +506,9 @@ _gnutls_handshake_verify_cert_vrfy12 (gnutls_session_t session,
  */
 int
 _gnutls_handshake_verify_cert_vrfy (gnutls_session_t session,
-				    gnutls_cert * cert,
-				    gnutls_datum_t * signature,
-				    gnutls_sign_algorithm_t sign_algo)
+                                    gnutls_cert * cert,
+                                    gnutls_datum_t * signature,
+                                    gnutls_sign_algorithm_t sign_algo)
 {
   int ret;
   opaque concat[MAX_SIG_SIZE];
@@ -552,10 +521,10 @@ _gnutls_handshake_verify_cert_vrfy (gnutls_session_t session,
       HANDSHAKE_MAC_TYPE_12)
     {
       return _gnutls_handshake_verify_cert_vrfy12 (session, cert, signature,
-						   sign_algo);
+                                                   sign_algo);
     }
   else if (session->security_parameters.handshake_mac_handle_type !=
-	   HANDSHAKE_MAC_TYPE_10)
+           HANDSHAKE_MAC_TYPE_10)
     {
       gnutls_assert ();
       return GNUTLS_E_INTERNAL_ERROR;
@@ -563,7 +532,7 @@ _gnutls_handshake_verify_cert_vrfy (gnutls_session_t session,
 
   ret =
     _gnutls_hash_copy (&td_md5,
-		       &session->internals.handshake_mac_handle.tls10.md5);
+                       &session->internals.handshake_mac_handle.tls10.md5);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -572,7 +541,7 @@ _gnutls_handshake_verify_cert_vrfy (gnutls_session_t session,
 
   ret =
     _gnutls_hash_copy (&td_sha,
-		       &session->internals.handshake_mac_handle.tls10.sha);
+                       &session->internals.handshake_mac_handle.tls10.sha);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -584,19 +553,19 @@ _gnutls_handshake_verify_cert_vrfy (gnutls_session_t session,
     {
       ret = _gnutls_generate_master (session, 1);
       if (ret < 0)
-	{
-	  gnutls_assert ();
-	  return ret;
-	}
+        {
+          gnutls_assert ();
+          return ret;
+        }
 
       _gnutls_mac_deinit_ssl3_handshake (&td_md5, concat,
-					 session->
-					 security_parameters.master_secret,
-					 GNUTLS_MASTER_SIZE);
+                                         session->
+                                         security_parameters.master_secret,
+                                         GNUTLS_MASTER_SIZE);
       _gnutls_mac_deinit_ssl3_handshake (&td_sha, &concat[16],
-					 session->
-					 security_parameters.master_secret,
-					 GNUTLS_MASTER_SIZE);
+                                         session->
+                                         security_parameters.master_secret,
+                                         GNUTLS_MASTER_SIZE);
     }
   else
     {
@@ -605,11 +574,11 @@ _gnutls_handshake_verify_cert_vrfy (gnutls_session_t session,
     }
 
   dconcat.data = concat;
-  dconcat.size = 20 + 16;	/* md5+ sha */
+  dconcat.size = 20 + 16;       /* md5+ sha */
 
   ret =
-    _gnutls_verify_sig (cert, &dconcat, signature, 16,
-			cert->subject_pk_algorithm);
+    verify_tls_hash (ver, cert, &dconcat, signature, 16,
+                        cert->subject_pk_algorithm);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -624,8 +593,8 @@ _gnutls_handshake_verify_cert_vrfy (gnutls_session_t session,
  */
 static int
 _gnutls_handshake_sign_cert_vrfy12 (gnutls_session_t session,
-				    gnutls_cert * cert, gnutls_privkey_t pkey,
-				    gnutls_datum_t * signature)
+                                    gnutls_cert * cert, gnutls_privkey_t pkey,
+                                    gnutls_datum_t * signature)
 {
   gnutls_datum_t dconcat;
   int ret;
@@ -649,29 +618,29 @@ _gnutls_handshake_sign_cert_vrfy12 (gnutls_session_t session,
       handshake_td = &session->internals.handshake_mac_handle.tls12.sha256;
       hash_algo = handshake_td->algorithm;
       sign_algo =
-	_gnutls_x509_pk_to_sign (cert->subject_pk_algorithm, hash_algo);
+        _gnutls_x509_pk_to_sign (cert->subject_pk_algorithm, hash_algo);
       if (sign_algo == GNUTLS_SIGN_UNKNOWN)
-	{
-	  gnutls_assert ();
-	  return GNUTLS_E_UNSUPPORTED_SIGNATURE_ALGORITHM;
-	}
+        {
+          gnutls_assert ();
+          return GNUTLS_E_UNSUPPORTED_SIGNATURE_ALGORITHM;
+        }
 
       ret = _gnutls_session_sign_algo_requested (session, sign_algo);
       if (ret < 0)
-	{
-	  gnutls_assert ();
-	  _gnutls_x509_log
-	    ("Server did not allow either '%s' or '%s' for signing\n",
-	     gnutls_mac_get_name (hash_algo),
-	     gnutls_mac_get_name (session->internals.handshake_mac_handle.
-				  tls12.sha1.algorithm));
-	  return ret;
-	}
+        {
+          gnutls_assert ();
+          _gnutls_x509_log
+            ("Server did not allow either '%s' or '%s' for signing\n",
+             gnutls_mac_get_name (hash_algo),
+             gnutls_mac_get_name (session->internals.handshake_mac_handle.
+                                  tls12.sha1.algorithm));
+          return ret;
+        }
     }
 
   _gnutls_x509_log ("sign handshake cert vrfy: picked %s with %s\n",
-		    gnutls_sign_algorithm_get_name (sign_algo),
-		    gnutls_mac_get_name (hash_algo));
+                    gnutls_sign_algorithm_get_name (sign_algo),
+                    gnutls_mac_get_name (hash_algo));
 
   ret = _gnutls_hash_copy (&td, handshake_td);
   if (ret < 0)
@@ -685,7 +654,7 @@ _gnutls_handshake_sign_cert_vrfy12 (gnutls_session_t session,
   dconcat.data = concat;
   dconcat.size = _gnutls_hash_get_algo_len (hash_algo);
 
-  ret = _gnutls_tls_sign (session, cert, pkey, &dconcat, signature);
+  ret = sign_tls_hash (session, hash_algo, cert, pkey, &dconcat, signature);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -706,8 +675,8 @@ _gnutls_handshake_sign_cert_vrfy12 (gnutls_session_t session,
  */
 int
 _gnutls_handshake_sign_cert_vrfy (gnutls_session_t session,
-				  gnutls_cert * cert, gnutls_privkey_t pkey,
-				  gnutls_datum_t * signature)
+                                  gnutls_cert * cert, gnutls_privkey_t pkey,
+                                  gnutls_datum_t * signature)
 {
   gnutls_datum_t dconcat;
   int ret;
@@ -720,10 +689,10 @@ _gnutls_handshake_sign_cert_vrfy (gnutls_session_t session,
       HANDSHAKE_MAC_TYPE_12)
     {
       return _gnutls_handshake_sign_cert_vrfy12 (session, cert, pkey,
-						 signature);
+                                                 signature);
     }
   else if (session->security_parameters.handshake_mac_handle_type !=
-	   HANDSHAKE_MAC_TYPE_10)
+           HANDSHAKE_MAC_TYPE_10)
     {
       gnutls_assert ();
       return GNUTLS_E_INTERNAL_ERROR;
@@ -731,7 +700,7 @@ _gnutls_handshake_sign_cert_vrfy (gnutls_session_t session,
 
   ret =
     _gnutls_hash_copy (&td_sha,
-		       &session->internals.handshake_mac_handle.tls10.sha);
+                       &session->internals.handshake_mac_handle.tls10.sha);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -742,15 +711,15 @@ _gnutls_handshake_sign_cert_vrfy (gnutls_session_t session,
     {
       ret = _gnutls_generate_master (session, 1);
       if (ret < 0)
-	{
-	  gnutls_assert ();
-	  return ret;
-	}
+        {
+          gnutls_assert ();
+          return ret;
+        }
 
       _gnutls_mac_deinit_ssl3_handshake (&td_sha, &concat[16],
-					 session->
-					 security_parameters.master_secret,
-					 GNUTLS_MASTER_SIZE);
+                                         session->
+                                         security_parameters.master_secret,
+                                         GNUTLS_MASTER_SIZE);
     }
   else
     _gnutls_hash_deinit (&td_sha, &concat[16]);
@@ -759,22 +728,22 @@ _gnutls_handshake_sign_cert_vrfy (gnutls_session_t session,
     {
     case GNUTLS_PK_RSA:
       ret =
-	_gnutls_hash_copy (&td_md5,
-			   &session->internals.handshake_mac_handle.tls10.
-			   md5);
+        _gnutls_hash_copy (&td_md5,
+                           &session->internals.handshake_mac_handle.tls10.
+                           md5);
       if (ret < 0)
-	{
-	  gnutls_assert ();
-	  return ret;
-	}
+        {
+          gnutls_assert ();
+          return ret;
+        }
 
       if (ver == GNUTLS_SSL3)
-	_gnutls_mac_deinit_ssl3_handshake (&td_md5, concat,
-					   session->
-					   security_parameters.master_secret,
-					   GNUTLS_MASTER_SIZE);
+        _gnutls_mac_deinit_ssl3_handshake (&td_md5, concat,
+                                           session->
+                                           security_parameters.master_secret,
+                                           GNUTLS_MASTER_SIZE);
       else
-	_gnutls_hash_deinit (&td_md5, concat);
+        _gnutls_hash_deinit (&td_md5, concat);
 
       dconcat.data = concat;
       dconcat.size = 36;
@@ -788,7 +757,7 @@ _gnutls_handshake_sign_cert_vrfy (gnutls_session_t session,
       gnutls_assert ();
       return GNUTLS_E_INTERNAL_ERROR;
     }
-  ret = _gnutls_tls_sign (session, cert, pkey, &dconcat, signature);
+  ret = sign_tls_hash (session, GNUTLS_DIG_NULL, cert, pkey, &dconcat, signature);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -796,3 +765,170 @@ _gnutls_handshake_sign_cert_vrfy (gnutls_session_t session,
 
   return ret;
 }
+
+int
+pk_hash_data (gnutls_pk_algorithm_t pk, gnutls_digest_algorithm_t hash,
+              bigint_t * params,
+              const gnutls_datum_t * data, gnutls_datum_t * digest)
+{
+  int ret;
+
+  switch (pk)
+    {
+    case GNUTLS_PK_RSA:
+      break;
+    case GNUTLS_PK_DSA:
+      if (params && hash != _gnutls_dsa_q_to_hash (params[1]))
+        {
+          gnutls_assert ();
+          return GNUTLS_E_INVALID_REQUEST;
+        }
+      break;
+    default:
+      gnutls_assert ();
+      return GNUTLS_E_INVALID_REQUEST;
+    }
+
+  digest->size = _gnutls_hash_get_algo_len (hash);
+  digest->data = gnutls_malloc (digest->size);
+  if (digest->data == NULL)
+    {
+      gnutls_assert ();
+      return GNUTLS_E_MEMORY_ERROR;
+    }
+
+  ret = _gnutls_hash_fast (hash, data->data, data->size, digest->data);
+  if (ret < 0)
+    {
+      gnutls_assert ();
+      goto cleanup;
+    }
+
+  return 0;
+
+cleanup:
+  gnutls_free (digest->data);
+  return ret;
+}
+
+/* Writes the digest information and the digest in a DER encoded
+ * structure. The digest info is allocated and stored into the info structure.
+ */
+static int
+encode_ber_digest_info (gnutls_digest_algorithm_t hash,
+                        const gnutls_datum_t * digest,
+                        gnutls_datum_t * output)
+{
+  ASN1_TYPE dinfo = ASN1_TYPE_EMPTY;
+  int result;
+  const char *algo;
+  opaque *tmp_output;
+  int tmp_output_size;
+
+  algo = _gnutls_x509_mac_to_oid ((gnutls_mac_algorithm_t) hash);
+  if (algo == NULL)
+    {
+      gnutls_assert ();
+      _gnutls_x509_log ("Hash algorithm: %d\n", hash);
+      return GNUTLS_E_UNKNOWN_PK_ALGORITHM;
+    }
+
+  if ((result = asn1_create_element (_gnutls_get_gnutls_asn (),
+                                     "GNUTLS.DigestInfo",
+                                     &dinfo)) != ASN1_SUCCESS)
+    {
+      gnutls_assert ();
+      return _gnutls_asn2err (result);
+    }
+
+  result = asn1_write_value (dinfo, "digestAlgorithm.algorithm", algo, 1);
+  if (result != ASN1_SUCCESS)
+    {
+      gnutls_assert ();
+      asn1_delete_structure (&dinfo);
+      return _gnutls_asn2err (result);
+    }
+
+  /* Write an ASN.1 NULL in the parameters field.  This matches RFC
+     3279 and RFC 4055, although is arguable incorrect from a historic
+     perspective (see those documents for more information).
+     Regardless of what is correct, this appears to be what most
+     implementations do.  */
+  result = asn1_write_value (dinfo, "digestAlgorithm.parameters",
+                             ASN1_NULL, ASN1_NULL_SIZE);
+  if (result != ASN1_SUCCESS)
+    {
+      gnutls_assert ();
+      asn1_delete_structure (&dinfo);
+      return _gnutls_asn2err (result);
+    }
+
+  result = asn1_write_value (dinfo, "digest", digest->data, digest->size);
+  if (result != ASN1_SUCCESS)
+    {
+      gnutls_assert ();
+      asn1_delete_structure (&dinfo);
+      return _gnutls_asn2err (result);
+    }
+
+  tmp_output_size = 0;
+  asn1_der_coding (dinfo, "", NULL, &tmp_output_size, NULL);
+
+  tmp_output = gnutls_malloc (tmp_output_size);
+  if (output->data == NULL)
+    {
+      gnutls_assert ();
+      asn1_delete_structure (&dinfo);
+      return GNUTLS_E_MEMORY_ERROR;
+    }
+
+  result = asn1_der_coding (dinfo, "", tmp_output, &tmp_output_size, NULL);
+  if (result != ASN1_SUCCESS)
+    {
+      gnutls_assert ();
+      asn1_delete_structure (&dinfo);
+      return _gnutls_asn2err (result);
+    }
+
+  asn1_delete_structure (&dinfo);
+
+  output->size = tmp_output_size;
+  output->data = tmp_output;
+
+  return 0;
+}
+
+/* 
+ * This function will do RSA PKCS #1 1.5 encoding
+ * on the given digest. The given digest must be allocated
+ * and will be freed if replacement is required.
+ */
+int
+pk_prepare_hash (gnutls_pk_algorithm_t pk,
+                 gnutls_digest_algorithm_t hash, gnutls_datum_t * digest)
+{
+  int ret;
+  gnutls_datum old_digest = { digest->data, digest->size };
+
+  switch (pk)
+    {
+    case GNUTLS_PK_RSA:
+      /* Encode the digest as a DigestInfo
+       */
+      if ((ret = encode_ber_digest_info (hash, &old_digest, digest)) != 0)
+        {
+          gnutls_assert ();
+          return ret;
+        }
+
+      _gnutls_free_datum (&old_digest);
+      break;
+    case GNUTLS_PK_DSA:
+      break;
+    default:
+      gnutls_assert ();
+      return GNUTLS_E_UNIMPLEMENTED_FEATURE;
+    }
+
+  return 0;
+}
