@@ -34,7 +34,6 @@
 #include <x509_b64.h>
 #include <x509_int.h>
 #include <gnutls_pk.h>
-#include <sign.h>
 #include <gnutls_mpi.h>
 
 static int _gnutls_asn1_encode_rsa (ASN1_TYPE * c2, bigint_t * params);
@@ -57,7 +56,7 @@ gnutls_x509_privkey_init (gnutls_x509_privkey_t * key)
     {
       (*key)->key = ASN1_TYPE_EMPTY;
       (*key)->pk_algorithm = GNUTLS_PK_UNKNOWN;
-      return 0;			/* success */
+      return 0;                 /* success */
     }
 
   return GNUTLS_E_MEMORY_ERROR;
@@ -109,7 +108,7 @@ gnutls_x509_privkey_cpy (gnutls_x509_privkey_t dst, gnutls_x509_privkey_t src)
     {
       dst->params[i] = _gnutls_mpi_copy (src->params[i]);
       if (dst->params[i] == NULL)
-	return GNUTLS_E_MEMORY_ERROR;
+        return GNUTLS_E_MEMORY_ERROR;
     }
 
   dst->params_size = src->params_size;
@@ -119,27 +118,27 @@ gnutls_x509_privkey_cpy (gnutls_x509_privkey_t dst, gnutls_x509_privkey_t src)
   if (!src->crippled)
     {
       switch (dst->pk_algorithm)
-	{
-	case GNUTLS_PK_DSA:
-	  ret = _gnutls_asn1_encode_dsa (&dst->key, dst->params);
-	  if (ret < 0)
-	    {
-	      gnutls_assert ();
-	      return ret;
-	    }
-	  break;
-	case GNUTLS_PK_RSA:
-	  ret = _gnutls_asn1_encode_rsa (&dst->key, dst->params);
-	  if (ret < 0)
-	    {
-	      gnutls_assert ();
-	      return ret;
-	    }
-	  break;
-	default:
-	  gnutls_assert ();
-	  return GNUTLS_E_INVALID_REQUEST;
-	}
+        {
+        case GNUTLS_PK_DSA:
+          ret = _gnutls_asn1_encode_dsa (&dst->key, dst->params);
+          if (ret < 0)
+            {
+              gnutls_assert ();
+              return ret;
+            }
+          break;
+        case GNUTLS_PK_RSA:
+          ret = _gnutls_asn1_encode_rsa (&dst->key, dst->params);
+          if (ret < 0)
+            {
+              gnutls_assert ();
+              return ret;
+            }
+          break;
+        default:
+          gnutls_assert ();
+          return GNUTLS_E_INVALID_REQUEST;
+        }
     }
 
   return 0;
@@ -150,7 +149,7 @@ gnutls_x509_privkey_cpy (gnutls_x509_privkey_t dst, gnutls_x509_privkey_t src)
  */
 ASN1_TYPE
 _gnutls_privkey_decode_pkcs1_rsa_key (const gnutls_datum_t * raw_key,
-				      gnutls_x509_privkey_t pkey)
+                                      gnutls_x509_privkey_t pkey)
 {
   int result;
   ASN1_TYPE pkey_asn;
@@ -161,8 +160,8 @@ _gnutls_privkey_decode_pkcs1_rsa_key (const gnutls_datum_t * raw_key,
 
   if ((result =
        asn1_create_element (_gnutls_get_gnutls_asn (),
-			    "GNUTLS.RSAPrivateKey",
-			    &pkey_asn)) != ASN1_SUCCESS)
+                            "GNUTLS.RSAPrivateKey",
+                            &pkey_asn)) != ASN1_SUCCESS)
     {
       gnutls_assert ();
       return NULL;
@@ -176,7 +175,7 @@ _gnutls_privkey_decode_pkcs1_rsa_key (const gnutls_datum_t * raw_key,
     }
 
   if ((result = _gnutls_x509_read_int (pkey_asn, "modulus",
-				       &pk_params.params[0])) < 0)
+                                       &pk_params.params[0])) < 0)
     {
       gnutls_assert ();
       goto error;
@@ -184,7 +183,7 @@ _gnutls_privkey_decode_pkcs1_rsa_key (const gnutls_datum_t * raw_key,
 
   if ((result =
        _gnutls_x509_read_int (pkey_asn, "publicExponent",
-			      &pk_params.params[1])) < 0)
+                              &pk_params.params[1])) < 0)
     {
       gnutls_assert ();
       goto error;
@@ -192,42 +191,42 @@ _gnutls_privkey_decode_pkcs1_rsa_key (const gnutls_datum_t * raw_key,
 
   if ((result =
        _gnutls_x509_read_int (pkey_asn, "privateExponent",
-			      &pk_params.params[2])) < 0)
+                              &pk_params.params[2])) < 0)
     {
       gnutls_assert ();
       goto error;
     }
 
   if ((result = _gnutls_x509_read_int (pkey_asn, "prime1",
-				       &pk_params.params[3])) < 0)
+                                       &pk_params.params[3])) < 0)
     {
       gnutls_assert ();
       goto error;
     }
 
   if ((result = _gnutls_x509_read_int (pkey_asn, "prime2",
-				       &pk_params.params[4])) < 0)
+                                       &pk_params.params[4])) < 0)
     {
       gnutls_assert ();
       goto error;
     }
 
   if ((result = _gnutls_x509_read_int (pkey_asn, "coefficient",
-				       &pk_params.params[5])) < 0)
+                                       &pk_params.params[5])) < 0)
     {
       gnutls_assert ();
       goto error;
     }
 
   if ((result = _gnutls_x509_read_int (pkey_asn, "exponent1",
-				       &pk_params.params[6])) < 0)
+                                       &pk_params.params[6])) < 0)
     {
       gnutls_assert ();
       goto error;
     }
 
   if ((result = _gnutls_x509_read_int (pkey_asn, "exponent2",
-				       &pk_params.params[7])) < 0)
+                                       &pk_params.params[7])) < 0)
     {
       gnutls_assert ();
       goto error;
@@ -268,8 +267,8 @@ decode_dsa_key (const gnutls_datum_t * raw_key, gnutls_x509_privkey_t pkey)
 
   if ((result =
        asn1_create_element (_gnutls_get_gnutls_asn (),
-			    "GNUTLS.DSAPrivateKey",
-			    &dsa_asn)) != ASN1_SUCCESS)
+                            "GNUTLS.DSAPrivateKey",
+                            &dsa_asn)) != ASN1_SUCCESS)
     {
       gnutls_assert ();
       return NULL;
@@ -307,7 +306,7 @@ decode_dsa_key (const gnutls_datum_t * raw_key, gnutls_x509_privkey_t pkey)
     }
 
   if ((result = _gnutls_x509_read_int (dsa_asn, "priv",
-				       &pkey->params[4])) < 0)
+                                       &pkey->params[4])) < 0)
     {
       gnutls_assert ();
       goto error;
@@ -349,8 +348,8 @@ error:
  **/
 int
 gnutls_x509_privkey_import (gnutls_x509_privkey_t key,
-			    const gnutls_datum_t * data,
-			    gnutls_x509_crt_fmt_t format)
+                            const gnutls_datum_t * data,
+                            gnutls_x509_crt_fmt_t format)
 {
   int result = 0, need_free = 0;
   gnutls_datum_t _data;
@@ -374,28 +373,28 @@ gnutls_x509_privkey_import (gnutls_x509_privkey_t key,
 
       /* Try the first header */
       result =
-	_gnutls_fbase64_decode (PEM_KEY_RSA, data->data, data->size, &out);
+        _gnutls_fbase64_decode (PEM_KEY_RSA, data->data, data->size, &out);
 
       if (result >= 0)
-	key->pk_algorithm = GNUTLS_PK_RSA;
+        key->pk_algorithm = GNUTLS_PK_RSA;
 
       if (result == GNUTLS_E_BASE64_UNEXPECTED_HEADER_ERROR)
-	{
-	  /* try for the second header */
-	  result =
-	    _gnutls_fbase64_decode (PEM_KEY_DSA, data->data, data->size,
-				    &out);
-	  if (result <= 0)
-	    {
-	      if (result == 0)
-		result = GNUTLS_E_INTERNAL_ERROR;
-	      gnutls_assert ();
+        {
+          /* try for the second header */
+          result =
+            _gnutls_fbase64_decode (PEM_KEY_DSA, data->data, data->size,
+                                    &out);
+          if (result <= 0)
+            {
+              if (result == 0)
+                result = GNUTLS_E_INTERNAL_ERROR;
+              gnutls_assert ();
 
-	      goto failover;
-	    }
+              goto failover;
+            }
 
-	  key->pk_algorithm = GNUTLS_PK_DSA;
-	}
+          key->pk_algorithm = GNUTLS_PK_DSA;
+        }
 
       _data.data = out;
       _data.size = result;
@@ -407,13 +406,13 @@ gnutls_x509_privkey_import (gnutls_x509_privkey_t key,
     {
       key->key = _gnutls_privkey_decode_pkcs1_rsa_key (&_data, key);
       if (key->key == NULL)
-	gnutls_assert ();
+        gnutls_assert ();
     }
   else if (key->pk_algorithm == GNUTLS_PK_DSA)
     {
       key->key = decode_dsa_key (&_data, key);
       if (key->key == NULL)
-	gnutls_assert ();
+        gnutls_assert ();
     }
   else
     {
@@ -424,12 +423,12 @@ gnutls_x509_privkey_import (gnutls_x509_privkey_t key,
       key->key = _gnutls_privkey_decode_pkcs1_rsa_key (&_data, key);
 
       if (key->key == NULL)
-	{
-	  key->pk_algorithm = GNUTLS_PK_DSA;
-	  key->key = decode_dsa_key (&_data, key);
-	  if (key->key == NULL)
-	    gnutls_assert ();
-	}
+        {
+          key->pk_algorithm = GNUTLS_PK_DSA;
+          key->key = decode_dsa_key (&_data, key);
+          if (key->key == NULL)
+            gnutls_assert ();
+        }
     }
 
   if (key->key == NULL)
@@ -454,7 +453,7 @@ failover:
     {
       _gnutls_debug_log ("Falling back to PKCS #8 key decoding\n");
       result = gnutls_x509_privkey_import_pkcs8 (key, data, format,
-						 NULL, GNUTLS_PKCS_PLAIN);
+                                                 NULL, GNUTLS_PKCS_PLAIN);
     }
 #endif
 
@@ -488,15 +487,15 @@ failover:
  **/
 int
 gnutls_x509_privkey_import_rsa_raw (gnutls_x509_privkey_t key,
-				    const gnutls_datum_t * m,
-				    const gnutls_datum_t * e,
-				    const gnutls_datum_t * d,
-				    const gnutls_datum_t * p,
-				    const gnutls_datum_t * q,
-				    const gnutls_datum_t * u)
+                                    const gnutls_datum_t * m,
+                                    const gnutls_datum_t * e,
+                                    const gnutls_datum_t * d,
+                                    const gnutls_datum_t * p,
+                                    const gnutls_datum_t * q,
+                                    const gnutls_datum_t * u)
 {
   return gnutls_x509_privkey_import_rsa_raw2 (key, m, e, d, p, q, u, NULL,
-					      NULL);
+                                              NULL);
 }
 
 /**
@@ -518,14 +517,14 @@ gnutls_x509_privkey_import_rsa_raw (gnutls_x509_privkey_t key,
  **/
 int
 gnutls_x509_privkey_import_rsa_raw2 (gnutls_x509_privkey_t key,
-				     const gnutls_datum_t * m,
-				     const gnutls_datum_t * e,
-				     const gnutls_datum_t * d,
-				     const gnutls_datum_t * p,
-				     const gnutls_datum_t * q,
-				     const gnutls_datum_t * u,
-				     const gnutls_datum_t * e1,
-				     const gnutls_datum_t * e2)
+                                     const gnutls_datum_t * m,
+                                     const gnutls_datum_t * e,
+                                     const gnutls_datum_t * d,
+                                     const gnutls_datum_t * p,
+                                     const gnutls_datum_t * q,
+                                     const gnutls_datum_t * u,
+                                     const gnutls_datum_t * e1,
+                                     const gnutls_datum_t * e2)
 {
   int i = 0, ret;
   size_t siz = 0;
@@ -599,20 +598,20 @@ gnutls_x509_privkey_import_rsa_raw2 (gnutls_x509_privkey_t key,
     {
       siz = e1->size;
       if (_gnutls_mpi_scan_nz (&key->params[6], e1->data, siz))
-	{
-	  gnutls_assert ();
-	  FREE_RSA_PRIVATE_PARAMS;
-	  return GNUTLS_E_MPI_SCAN_FAILED;
-	}
+        {
+          gnutls_assert ();
+          FREE_RSA_PRIVATE_PARAMS;
+          return GNUTLS_E_MPI_SCAN_FAILED;
+        }
       key->params_size++;
 
       siz = e2->size;
       if (_gnutls_mpi_scan_nz (&key->params[7], e2->data, siz))
-	{
-	  gnutls_assert ();
-	  FREE_RSA_PRIVATE_PARAMS;
-	  return GNUTLS_E_MPI_SCAN_FAILED;
-	}
+        {
+          gnutls_assert ();
+          FREE_RSA_PRIVATE_PARAMS;
+          return GNUTLS_E_MPI_SCAN_FAILED;
+        }
       key->params_size++;
     }
 
@@ -641,11 +640,11 @@ gnutls_x509_privkey_import_rsa_raw2 (gnutls_x509_privkey_t key,
     {
       ret = _gnutls_asn1_encode_rsa (&key->key, key->params);
       if (ret < 0)
-	{
-	  gnutls_assert ();
-	  FREE_RSA_PRIVATE_PARAMS;
-	  return ret;
-	}
+        {
+          gnutls_assert ();
+          FREE_RSA_PRIVATE_PARAMS;
+          return ret;
+        }
     }
 
   key->params_size = RSA_PRIVATE_PARAMS;
@@ -673,11 +672,11 @@ gnutls_x509_privkey_import_rsa_raw2 (gnutls_x509_privkey_t key,
  **/
 int
 gnutls_x509_privkey_import_dsa_raw (gnutls_x509_privkey_t key,
-				    const gnutls_datum_t * p,
-				    const gnutls_datum_t * q,
-				    const gnutls_datum_t * g,
-				    const gnutls_datum_t * y,
-				    const gnutls_datum_t * x)
+                                    const gnutls_datum_t * p,
+                                    const gnutls_datum_t * q,
+                                    const gnutls_datum_t * g,
+                                    const gnutls_datum_t * y,
+                                    const gnutls_datum_t * x)
 {
   int i = 0, ret;
   size_t siz = 0;
@@ -732,11 +731,11 @@ gnutls_x509_privkey_import_dsa_raw (gnutls_x509_privkey_t key,
     {
       ret = _gnutls_asn1_encode_dsa (&key->key, key->params);
       if (ret < 0)
-	{
-	  gnutls_assert ();
-	  FREE_DSA_PRIVATE_PARAMS;
-	  return ret;
-	}
+        {
+          gnutls_assert ();
+          FREE_DSA_PRIVATE_PARAMS;
+          return ret;
+        }
     }
 
   key->params_size = DSA_PRIVATE_PARAMS;
@@ -793,8 +792,8 @@ gnutls_x509_privkey_get_pk_algorithm (gnutls_x509_privkey_t key)
  **/
 int
 gnutls_x509_privkey_export (gnutls_x509_privkey_t key,
-			    gnutls_x509_crt_fmt_t format, void *output_data,
-			    size_t * output_data_size)
+                            gnutls_x509_crt_fmt_t format, void *output_data,
+                            size_t * output_data_size)
 {
   const char *msg;
   int ret;
@@ -813,34 +812,34 @@ gnutls_x509_privkey_export (gnutls_x509_privkey_t key,
     msg = NULL;
 
   if (key->crippled)
-    {				/* encode the parameters on the fly.
-				 */
+    {                           /* encode the parameters on the fly.
+                                 */
       switch (key->pk_algorithm)
-	{
-	case GNUTLS_PK_DSA:
-	  ret = _gnutls_asn1_encode_dsa (&key->key, key->params);
-	  if (ret < 0)
-	    {
-	      gnutls_assert ();
-	      return ret;
-	    }
-	  break;
-	case GNUTLS_PK_RSA:
-	  ret = _gnutls_asn1_encode_rsa (&key->key, key->params);
-	  if (ret < 0)
-	    {
-	      gnutls_assert ();
-	      return ret;
-	    }
-	  break;
-	default:
-	  gnutls_assert ();
-	  return GNUTLS_E_INVALID_REQUEST;
-	}
+        {
+        case GNUTLS_PK_DSA:
+          ret = _gnutls_asn1_encode_dsa (&key->key, key->params);
+          if (ret < 0)
+            {
+              gnutls_assert ();
+              return ret;
+            }
+          break;
+        case GNUTLS_PK_RSA:
+          ret = _gnutls_asn1_encode_rsa (&key->key, key->params);
+          if (ret < 0)
+            {
+              gnutls_assert ();
+              return ret;
+            }
+          break;
+        default:
+          gnutls_assert ();
+          return GNUTLS_E_INVALID_REQUEST;
+        }
     }
 
   return _gnutls_x509_export_int (key->key, format, msg,
-				  output_data, output_data_size);
+                                  output_data, output_data_size);
 }
 
 /**
@@ -861,12 +860,12 @@ gnutls_x509_privkey_sec_param (gnutls_x509_privkey_t key)
   switch (key->pk_algorithm)
     {
     case GNUTLS_PK_RSA:
-      ret = gnutls_pk_bits_to_sec_param (GNUTLS_PK_RSA, _gnutls_mpi_get_nbits (key->params[0]	/*m */
-					 ));
+      ret = gnutls_pk_bits_to_sec_param (GNUTLS_PK_RSA, _gnutls_mpi_get_nbits (key->params[0]   /*m */
+                                         ));
       break;
     case GNUTLS_PK_DSA:
-      ret = gnutls_pk_bits_to_sec_param (GNUTLS_PK_DSA, _gnutls_mpi_get_nbits (key->params[0]	/*p */
-					 ));
+      ret = gnutls_pk_bits_to_sec_param (GNUTLS_PK_DSA, _gnutls_mpi_get_nbits (key->params[0]   /*p */
+                                         ));
       break;
     default:
       ret = GNUTLS_SEC_PARAM_UNKNOWN;
@@ -894,13 +893,13 @@ gnutls_x509_privkey_sec_param (gnutls_x509_privkey_t key)
  **/
 int
 gnutls_x509_privkey_export_rsa_raw (gnutls_x509_privkey_t key,
-				    gnutls_datum_t * m, gnutls_datum_t * e,
-				    gnutls_datum_t * d, gnutls_datum_t * p,
-				    gnutls_datum_t * q, gnutls_datum_t * u)
+                                    gnutls_datum_t * m, gnutls_datum_t * e,
+                                    gnutls_datum_t * d, gnutls_datum_t * p,
+                                    gnutls_datum_t * q, gnutls_datum_t * u)
 {
 
   return gnutls_x509_privkey_export_rsa_raw2 (key, m, e, d, p, q, u, NULL,
-					      NULL);
+                                              NULL);
 }
 
 /**
@@ -924,10 +923,10 @@ gnutls_x509_privkey_export_rsa_raw (gnutls_x509_privkey_t key,
  **/
 int
 gnutls_x509_privkey_export_rsa_raw2 (gnutls_x509_privkey_t key,
-				     gnutls_datum_t * m, gnutls_datum_t * e,
-				     gnutls_datum_t * d, gnutls_datum_t * p,
-				     gnutls_datum_t * q, gnutls_datum_t * u,
-				     gnutls_datum_t * e1, gnutls_datum_t * e2)
+                                     gnutls_datum_t * m, gnutls_datum_t * e,
+                                     gnutls_datum_t * d, gnutls_datum_t * p,
+                                     gnutls_datum_t * q, gnutls_datum_t * u,
+                                     gnutls_datum_t * e1, gnutls_datum_t * e2)
 {
   int ret;
   gnutls_pk_params_st pk_params;
@@ -957,7 +956,7 @@ gnutls_x509_privkey_export_rsa_raw2 (gnutls_x509_privkey_t key,
       goto error;
     }
 
-  ret = _gnutls_mpi_dprint (pk_params.params[0], m);
+  ret = _gnutls_mpi_dprint_lz (pk_params.params[0], m);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -965,7 +964,7 @@ gnutls_x509_privkey_export_rsa_raw2 (gnutls_x509_privkey_t key,
     }
 
   /* E */
-  ret = _gnutls_mpi_dprint (pk_params.params[1], e);
+  ret = _gnutls_mpi_dprint_lz (pk_params.params[1], e);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -973,7 +972,7 @@ gnutls_x509_privkey_export_rsa_raw2 (gnutls_x509_privkey_t key,
     }
 
   /* D */
-  ret = _gnutls_mpi_dprint (pk_params.params[2], d);
+  ret = _gnutls_mpi_dprint_lz (pk_params.params[2], d);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -981,7 +980,7 @@ gnutls_x509_privkey_export_rsa_raw2 (gnutls_x509_privkey_t key,
     }
 
   /* P */
-  ret = _gnutls_mpi_dprint (pk_params.params[3], p);
+  ret = _gnutls_mpi_dprint_lz (pk_params.params[3], p);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -989,7 +988,7 @@ gnutls_x509_privkey_export_rsa_raw2 (gnutls_x509_privkey_t key,
     }
 
   /* Q */
-  ret = _gnutls_mpi_dprint (pk_params.params[4], q);
+  ret = _gnutls_mpi_dprint_lz (pk_params.params[4], q);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -997,7 +996,7 @@ gnutls_x509_privkey_export_rsa_raw2 (gnutls_x509_privkey_t key,
     }
 
   /* U */
-  ret = _gnutls_mpi_dprint (key->params[5], u);
+  ret = _gnutls_mpi_dprint_lz (key->params[5], u);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -1007,23 +1006,23 @@ gnutls_x509_privkey_export_rsa_raw2 (gnutls_x509_privkey_t key,
   /* E1 */
   if (e1)
     {
-      ret = _gnutls_mpi_dprint (key->params[6], e1);
+      ret = _gnutls_mpi_dprint_lz (key->params[6], e1);
       if (ret < 0)
-	{
-	  gnutls_assert ();
-	  goto error;
-	}
+        {
+          gnutls_assert ();
+          goto error;
+        }
     }
 
   /* E2 */
   if (e2)
     {
-      ret = _gnutls_mpi_dprint (key->params[7], e2);
+      ret = _gnutls_mpi_dprint_lz (key->params[7], e2);
       if (ret < 0)
-	{
-	  gnutls_assert ();
-	  goto error;
-	}
+        {
+          gnutls_assert ();
+          goto error;
+        }
     }
 
   gnutls_pk_params_release (&pk_params);
@@ -1059,9 +1058,9 @@ error:
  **/
 int
 gnutls_x509_privkey_export_dsa_raw (gnutls_x509_privkey_t key,
-				    gnutls_datum_t * p, gnutls_datum_t * q,
-				    gnutls_datum_t * g, gnutls_datum_t * y,
-				    gnutls_datum_t * x)
+                                    gnutls_datum_t * p, gnutls_datum_t * q,
+                                    gnutls_datum_t * g, gnutls_datum_t * y,
+                                    gnutls_datum_t * x)
 {
   int ret;
 
@@ -1072,7 +1071,7 @@ gnutls_x509_privkey_export_dsa_raw (gnutls_x509_privkey_t key,
     }
 
   /* P */
-  ret = _gnutls_mpi_dprint (key->params[0], p);
+  ret = _gnutls_mpi_dprint_lz (key->params[0], p);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -1080,7 +1079,7 @@ gnutls_x509_privkey_export_dsa_raw (gnutls_x509_privkey_t key,
     }
 
   /* Q */
-  ret = _gnutls_mpi_dprint (key->params[1], q);
+  ret = _gnutls_mpi_dprint_lz (key->params[1], q);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -1090,7 +1089,7 @@ gnutls_x509_privkey_export_dsa_raw (gnutls_x509_privkey_t key,
 
 
   /* G */
-  ret = _gnutls_mpi_dprint (key->params[2], g);
+  ret = _gnutls_mpi_dprint_lz (key->params[2], g);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -1101,7 +1100,7 @@ gnutls_x509_privkey_export_dsa_raw (gnutls_x509_privkey_t key,
 
 
   /* Y */
-  ret = _gnutls_mpi_dprint (key->params[3], y);
+  ret = _gnutls_mpi_dprint_lz (key->params[3], y);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -1112,7 +1111,7 @@ gnutls_x509_privkey_export_dsa_raw (gnutls_x509_privkey_t key,
     }
 
   /* X */
-  ret = _gnutls_mpi_dprint (key->params[4], x);
+  ret = _gnutls_mpi_dprint_lz (key->params[4], x);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -1242,7 +1241,7 @@ _gnutls_asn1_encode_rsa (ASN1_TYPE * c2, bigint_t * params)
   /* Write PRIME 
    */
   if ((result = asn1_write_value (*c2, "modulus",
-				  m.data, m.size)) != ASN1_SUCCESS)
+                                  m.data, m.size)) != ASN1_SUCCESS)
     {
       gnutls_assert ();
       result = _gnutls_asn2err (result);
@@ -1250,7 +1249,7 @@ _gnutls_asn1_encode_rsa (ASN1_TYPE * c2, bigint_t * params)
     }
 
   if ((result = asn1_write_value (*c2, "publicExponent",
-				  e.data, e.size)) != ASN1_SUCCESS)
+                                  e.data, e.size)) != ASN1_SUCCESS)
     {
       gnutls_assert ();
       result = _gnutls_asn2err (result);
@@ -1258,7 +1257,7 @@ _gnutls_asn1_encode_rsa (ASN1_TYPE * c2, bigint_t * params)
     }
 
   if ((result = asn1_write_value (*c2, "privateExponent",
-				  d.data, d.size)) != ASN1_SUCCESS)
+                                  d.data, d.size)) != ASN1_SUCCESS)
     {
       gnutls_assert ();
       result = _gnutls_asn2err (result);
@@ -1266,7 +1265,7 @@ _gnutls_asn1_encode_rsa (ASN1_TYPE * c2, bigint_t * params)
     }
 
   if ((result = asn1_write_value (*c2, "prime1",
-				  p.data, p.size)) != ASN1_SUCCESS)
+                                  p.data, p.size)) != ASN1_SUCCESS)
     {
       gnutls_assert ();
       result = _gnutls_asn2err (result);
@@ -1274,7 +1273,7 @@ _gnutls_asn1_encode_rsa (ASN1_TYPE * c2, bigint_t * params)
     }
 
   if ((result = asn1_write_value (*c2, "prime2",
-				  q.data, q.size)) != ASN1_SUCCESS)
+                                  q.data, q.size)) != ASN1_SUCCESS)
     {
       gnutls_assert ();
       result = _gnutls_asn2err (result);
@@ -1282,7 +1281,7 @@ _gnutls_asn1_encode_rsa (ASN1_TYPE * c2, bigint_t * params)
     }
 
   if ((result = asn1_write_value (*c2, "coefficient",
-				  u.data, u.size)) != ASN1_SUCCESS)
+                                  u.data, u.size)) != ASN1_SUCCESS)
     {
       gnutls_assert ();
       result = _gnutls_asn2err (result);
@@ -1291,7 +1290,7 @@ _gnutls_asn1_encode_rsa (ASN1_TYPE * c2, bigint_t * params)
     }
 
   if ((result = asn1_write_value (*c2, "exponent1",
-				  exp1.data, exp1.size)) != ASN1_SUCCESS)
+                                  exp1.data, exp1.size)) != ASN1_SUCCESS)
     {
       gnutls_assert ();
       result = _gnutls_asn2err (result);
@@ -1299,7 +1298,7 @@ _gnutls_asn1_encode_rsa (ASN1_TYPE * c2, bigint_t * params)
     }
 
   if ((result = asn1_write_value (*c2, "exponent2",
-				  exp2.data, exp2.size)) != ASN1_SUCCESS)
+                                  exp2.data, exp2.size)) != ASN1_SUCCESS)
     {
       gnutls_assert ();
       result = _gnutls_asn2err (result);
@@ -1307,7 +1306,7 @@ _gnutls_asn1_encode_rsa (ASN1_TYPE * c2, bigint_t * params)
     }
 
   if ((result = asn1_write_value (*c2, "otherPrimeInfos",
-				  NULL, 0)) != ASN1_SUCCESS)
+                                  NULL, 0)) != ASN1_SUCCESS)
     {
       gnutls_assert ();
       result = _gnutls_asn2err (result);
@@ -1438,7 +1437,7 @@ _gnutls_asn1_encode_dsa (ASN1_TYPE * c2, bigint_t * params)
     }
 
   if ((result = asn1_write_value (*c2, "priv",
-				  x_data, size[4])) != ASN1_SUCCESS)
+                                  x_data, size[4])) != ASN1_SUCCESS)
     {
       gnutls_assert ();
       result = _gnutls_asn2err (result);
@@ -1481,8 +1480,8 @@ cleanup:
  **/
 int
 gnutls_x509_privkey_generate (gnutls_x509_privkey_t key,
-			      gnutls_pk_algorithm_t algo, unsigned int bits,
-			      unsigned int flags)
+                              gnutls_pk_algorithm_t algo, unsigned int bits,
+                              unsigned int flags)
 {
   int ret;
   unsigned int params_len = MAX_PRIV_PARAMS_SIZE;
@@ -1499,26 +1498,26 @@ gnutls_x509_privkey_generate (gnutls_x509_privkey_t key,
     case GNUTLS_PK_DSA:
       ret = _gnutls_dsa_generate_params (key->params, &params_len, bits);
       if (params_len != DSA_PRIVATE_PARAMS)
-	{
-	  gnutls_assert ();
-	  ret = GNUTLS_E_INTERNAL_ERROR;
-	}
+        {
+          gnutls_assert ();
+          ret = GNUTLS_E_INTERNAL_ERROR;
+        }
 
       if (ret < 0)
-	{
-	  gnutls_assert ();
-	  return ret;
-	}
+        {
+          gnutls_assert ();
+          return ret;
+        }
 
       if (!key->crippled)
-	{
-	  ret = _gnutls_asn1_encode_dsa (&key->key, key->params);
-	  if (ret < 0)
-	    {
-	      gnutls_assert ();
-	      goto cleanup;
-	    }
-	}
+        {
+          ret = _gnutls_asn1_encode_dsa (&key->key, key->params);
+          if (ret < 0)
+            {
+              gnutls_assert ();
+              goto cleanup;
+            }
+        }
       key->params_size = params_len;
       key->pk_algorithm = GNUTLS_PK_DSA;
 
@@ -1526,25 +1525,25 @@ gnutls_x509_privkey_generate (gnutls_x509_privkey_t key,
     case GNUTLS_PK_RSA:
       ret = _gnutls_rsa_generate_params (key->params, &params_len, bits);
       if (params_len != RSA_PRIVATE_PARAMS)
-	{
-	  gnutls_assert ();
-	  ret = GNUTLS_E_INTERNAL_ERROR;
-	}
+        {
+          gnutls_assert ();
+          ret = GNUTLS_E_INTERNAL_ERROR;
+        }
       if (ret < 0)
-	{
-	  gnutls_assert ();
-	  return ret;
-	}
+        {
+          gnutls_assert ();
+          return ret;
+        }
 
       if (!key->crippled)
-	{
-	  ret = _gnutls_asn1_encode_rsa (&key->key, key->params);
-	  if (ret < 0)
-	    {
-	      gnutls_assert ();
-	      goto cleanup;
-	    }
-	}
+        {
+          ret = _gnutls_asn1_encode_rsa (&key->key, key->params);
+          if (ret < 0)
+            {
+              gnutls_assert ();
+              goto cleanup;
+            }
+        }
 
       key->params_size = params_len;
       key->pk_algorithm = GNUTLS_PK_RSA;
@@ -1588,9 +1587,9 @@ cleanup:
  **/
 int
 gnutls_x509_privkey_get_key_id (gnutls_x509_privkey_t key,
-				unsigned int flags,
-				unsigned char *output_data,
-				size_t * output_data_size)
+                                unsigned int flags,
+                                unsigned char *output_data,
+                                size_t * output_data_size)
 {
   int result;
   digest_hd_st hd;
@@ -1612,23 +1611,23 @@ gnutls_x509_privkey_get_key_id (gnutls_x509_privkey_t key,
   if (key->pk_algorithm == GNUTLS_PK_RSA)
     {
       result =
-	_gnutls_x509_write_rsa_params (key->params, key->params_size, &der);
+        _gnutls_x509_write_rsa_params (key->params, key->params_size, &der);
       if (result < 0)
-	{
-	  gnutls_assert ();
-	  goto cleanup;
-	}
+        {
+          gnutls_assert ();
+          goto cleanup;
+        }
     }
   else if (key->pk_algorithm == GNUTLS_PK_DSA)
     {
       result =
-	_gnutls_x509_write_dsa_public_key (key->params,
-					   key->params_size, &der);
+        _gnutls_x509_write_dsa_public_key (key->params,
+                                           key->params_size, &der);
       if (result < 0)
-	{
-	  gnutls_assert ();
-	  goto cleanup;
-	}
+        {
+          gnutls_assert ();
+          goto cleanup;
+        }
     }
   else
     return GNUTLS_E_INTERNAL_ERROR;
@@ -1654,77 +1653,69 @@ cleanup:
 }
 
 
-/**
- * gnutls_x509_privkey_sign_data2:
- * @signer: Holds the key
- * @digest: should be MD5 or SHA1
- * @flags: should be 0 for now
- * @data: holds the data to be signed
- * @signature: will contain the signature allocate with gnutls_malloc()
+#ifdef ENABLE_PKI
+/*-
+ * _gnutls_x509_privkey_sign_hash2:
+ * @signer: Holds the signer's key
+ * @hash_algo: The hash algorithm used
+ * @hash_data: holds the data to be signed
+ * @signature: will contain newly allocated signature
+ * @flags: zero for now
  *
- * This function will sign the given data using a signature algorithm
+ * This function will sign the given hashed data using a signature algorithm
  * supported by the private key. Signature algorithms are always used
  * together with a hash functions.  Different hash functions may be
- * used for the RSA algorithm, but only SHA-1 for the DSA keys.
- *
- * If the buffer provided is not long enough to hold the output, then
- * *@signature_size is updated and %GNUTLS_E_SHORT_MEMORY_BUFFER will
- * be returned.
+ * used for the RSA algorithm, but only SHA-1,SHA-224 and SHA-256 
+ * for the DSA keys, depending on their bit size.
  *
  * Use gnutls_x509_crt_get_preferred_hash_algorithm() to determine
  * the hash algorithm.
  *
+ * The RSA algorithm is used in PKCS #1 v1.5 mode.
+ *
  * Returns: On success, %GNUTLS_E_SUCCESS is returned, otherwise a
  *   negative error value.
- **/
-int
-gnutls_x509_privkey_sign_data2 (gnutls_x509_privkey_t signer,
-				gnutls_digest_algorithm_t hash,
-				unsigned int flags,
-				const gnutls_datum_t * data,
-				gnutls_datum_t * signature)
+ -*/
+static int
+_gnutls_x509_privkey_sign_hash2 (gnutls_x509_privkey_t signer,
+                                gnutls_digest_algorithm_t hash_algo,
+                                unsigned int flags,
+                                const gnutls_datum_t * hash_data,
+                                gnutls_datum_t * signature)
 {
   int ret;
   gnutls_datum_t digest;
 
-  switch (signer->pk_algorithm)
+  digest.data = gnutls_malloc (hash_data->size);
+  if (digest.data == NULL)
     {
-    case GNUTLS_PK_RSA:
-      ret = pk_pkcs1_rsa_hash (hash, data, &digest);
-      if (ret < 0)
-	{
-	  gnutls_assert ();
-	  return ret;
-	}
-      break;
-    case GNUTLS_PK_DSA:
-      /* override hash for DSA */
-      ret =
-	pk_dsa_hash (_gnutls_dsa_q_to_hash (signer->params[1]), data,
-		     &digest);
-      if (ret < 0)
-	{
-	  gnutls_assert ();
-	  return ret;
-	}
-
-      break;
-    default:
       gnutls_assert ();
-      return GNUTLS_E_INTERNAL_ERROR;
+      return GNUTLS_E_MEMORY_ERROR;
+    }
+  digest.size = hash_data->size;
+  memcpy (digest.data, hash_data->data, digest.size);
+
+  ret = pk_prepare_hash (signer->pk_algorithm, hash_algo, &digest);
+  if (ret < 0)
+    {
+      gnutls_assert ();
+      goto cleanup;
     }
 
-  ret = gnutls_x509_privkey_sign_hash (signer, &digest, signature);
-  _gnutls_free_datum (&digest);
+  ret = _gnutls_soft_sign (signer->pk_algorithm, signer->params,
+                           signer->params_size, &digest, signature);
 
   if (ret < 0)
     {
       gnutls_assert ();
-      return ret;
+      goto cleanup;
     }
 
-  return 0;
+  ret = 0;
 
+cleanup:
+  _gnutls_free_datum (&digest);
+  return ret;
 }
 
 /**
@@ -1740,11 +1731,13 @@ gnutls_x509_privkey_sign_data2 (gnutls_x509_privkey_t signer,
  *
  * Returns: On success, %GNUTLS_E_SUCCESS is returned, otherwise a
  *   negative error value.
- **/
+ *
+ * Deprecated in: 2.12.0
+ */
 int
 gnutls_x509_privkey_sign_hash (gnutls_x509_privkey_t key,
-			       const gnutls_datum_t * hash,
-			       gnutls_datum_t * signature)
+                               const gnutls_datum_t * hash,
+                               gnutls_datum_t * signature)
 {
   int result;
 
@@ -1755,7 +1748,7 @@ gnutls_x509_privkey_sign_hash (gnutls_x509_privkey_t key,
     }
 
   result = _gnutls_soft_sign (key->pk_algorithm, key->params,
-			      key->params_size, hash, signature);
+                              key->params_size, hash, signature);
   if (result < 0)
     {
       gnutls_assert ();
@@ -1765,7 +1758,6 @@ gnutls_x509_privkey_sign_hash (gnutls_x509_privkey_t key,
   return 0;
 }
 
-#ifdef ENABLE_PKI
 /**
  * gnutls_x509_privkey_sign_data:
  * @key: Holds the key
@@ -1790,16 +1782,19 @@ gnutls_x509_privkey_sign_hash (gnutls_x509_privkey_t key,
  *
  * Returns: On success, %GNUTLS_E_SUCCESS is returned, otherwise a
  *   negative error value.
- **/
+ *
+ * Deprecated: Use gnutls_privkey_sign_data().
+ */
 int
 gnutls_x509_privkey_sign_data (gnutls_x509_privkey_t key,
-			       gnutls_digest_algorithm_t digest,
-			       unsigned int flags,
-			       const gnutls_datum_t * data,
-			       void *signature, size_t * signature_size)
+                               gnutls_digest_algorithm_t digest,
+                               unsigned int flags,
+                               const gnutls_datum_t * data,
+                               void *signature, size_t * signature_size)
 {
   int result;
   gnutls_datum_t sig = { NULL, 0 };
+  gnutls_datum_t hash;
 
   if (key == NULL)
     {
@@ -1807,13 +1802,25 @@ gnutls_x509_privkey_sign_data (gnutls_x509_privkey_t key,
       return GNUTLS_E_INVALID_REQUEST;
     }
 
-  result = gnutls_x509_privkey_sign_data2 (key, digest, flags, data, &sig);
+  result =
+    pk_hash_data (key->pk_algorithm, digest, key->params, data, &hash);
   if (result < 0)
     {
       gnutls_assert ();
       return result;
     }
 
+  result =
+    _gnutls_x509_privkey_sign_hash2 (key, digest, flags, &hash, signature);
+
+  _gnutls_free_datum(&hash);
+
+  if (result < 0)
+    {
+      gnutls_assert ();
+      return result;
+    }
+    
   if (*signature_size < sig.size)
     {
       *signature_size = sig.size;
@@ -1840,14 +1847,16 @@ gnutls_x509_privkey_sign_data (gnutls_x509_privkey_t key,
  * This function will verify the given signed data, using the
  * parameters in the private key.
  *
- * Returns: In case of a verification failure 0 is returned, and 1 on
- *   success.
- **/
+ * Returns: In case of a verification failure %GNUTLS_E_PK_SIG_VERIFY_FAILED 
+ * is returned, and a positive code on success.
+ *
+ * Deprecated: Use gnutls_pubkey_verify_data().
+ */
 int
 gnutls_x509_privkey_verify_data (gnutls_x509_privkey_t key,
-				 unsigned int flags,
-				 const gnutls_datum_t * data,
-				 const gnutls_datum_t * signature)
+                                 unsigned int flags,
+                                 const gnutls_datum_t * data,
+                                 const gnutls_datum_t * signature)
 {
   int result;
 
@@ -1861,7 +1870,7 @@ gnutls_x509_privkey_verify_data (gnutls_x509_privkey_t key,
   if (result < 0)
     {
       gnutls_assert ();
-      return 0;
+      return result;
     }
 
   return result;
@@ -1895,18 +1904,18 @@ gnutls_x509_privkey_fix (gnutls_x509_privkey_t key)
     case GNUTLS_PK_DSA:
       ret = _gnutls_asn1_encode_dsa (&key->key, key->params);
       if (ret < 0)
-	{
-	  gnutls_assert ();
-	  return ret;
-	}
+        {
+          gnutls_assert ();
+          return ret;
+        }
       break;
     case GNUTLS_PK_RSA:
       ret = _gnutls_asn1_encode_rsa (&key->key, key->params);
       if (ret < 0)
-	{
-	  gnutls_assert ();
-	  return ret;
-	}
+        {
+          gnutls_assert ();
+          return ret;
+        }
       break;
     default:
       gnutls_assert ();
